#include <string.h>
#include "inicon.h"
#include "./../../GEFEL/intools.h"

/**
  This constructor initializes attributes to zero values
*/
inicon::inicon()
{
  nlc  = 0L;
  ncon = 0L;
  con  = NULL;
}

/**
 This function reads data with initila condition description from
 the text preprocessor file given by the parameter in.

 Parameters :
 @param  in - pointer to opened text file with init. cond. data
 @param ndofn - number of init. cond values is equal to nodal dofs.
 @param lc - total number of load cases

 Returns :
 @retval 0 - on succes
 @retval 1 - invalid value of ndofn parameter
 @retval 2 - wrong load case id.
 @retval 3 - error reading condition value

*/
long inicon::read_prop(FILE *in, long ndofn, long lc)
{
  long i, err;

  if ((ndofn < 1) || (ndofn > 6))
    return(1);
  ncon = ndofn;
  con = new double[ncon];
  memset(con, 0, sizeof(*con)*ncon);

  getlong(in, nlc);
  if ((nlc < 1) || (nlc > lc))
    return(2);

  for (i = 0; i < ncon; i++)
  {
    err = getdouble(in, con[i]);
    if (err)
    {
      fprintf(stderr, "\n\nError reading initial condition in function\n");
      fprintf(stderr, " inicon::read_prop (inicon.cpp) \n");
      return(3);
    }
  }
  return(0);
}

/**
  This destructor deallocates used memory
*/
inicon::~inicon()
{
  ncon = 0L;
  delete [] con;
}
