#include "globprep.h"
#include "output.h"
#include <string.h>

long output(FILE *out, descrip *d)
{
  // Output data in mechtop format
  long err;
  long npropn, nprope;
  npropn = Npropn;
  nprope = Nprope;

  err = wr_nodes(out, npropn); // writing nodes, crsecs in the nodes and local systems
  fflush(out);
  if (err)
    return(7);

  err = wr_bocon(out, npropn); // writing boundary conditions
  fflush(out);
  if (err)
    return(7);

  err = wr_elements(out, nprope, d); // writing elements and code numbers on elements
  fflush(out);
  if (err)
    return(7);

  if (!(d->seq))
  {
    err = wr_globnodnum(out);
    if (err)
      return(7);
  }

//  err = wr_intpoints(out, nprope); // writing integration points of elements
                                   // with indeces of material parameters
  fflush(out);
  if (err)
    return(7);

  err = wr_materials(out); // writing material parameters
  fflush(out);
  if (err)
    return(7);

  if (Mp->stresscomp)
  {
    err = wr_auxpoint(out, nprope); // writing auxiliary points
    fflush(out);
    if (err)
      return(7);
    err = wr_ellcsys(out, nprope); // writing local coordinate systems in the auxiliary points
    fprintf(out, "\n");
    fflush(out);
    if (err)
      return(7);
  }
  if (Mp->straincomp)
  {
    err = wr_auxpoint(out, nprope); // writing auxiliary points
    fflush(out);
    if (err)
      return(7);
    err = wr_ellcsys(out, nprope); // writing local coordinate systems in the auxiliary points
    fprintf(out, "\n");
    fflush(out);
    if (err)
      return(7);
  }

  err = wr_crsecs(out);    // writing cross section parameters
  fflush(out);
  if (err)
    return(7);

  err = wr_load(out, npropn, nprope); // writing loads
  fflush(out);
  if (err)
    return(7);

  fclose(out);

  // deallocating used memory
  long i;
  for (i = 0; i < npropn; i++)
  {
    delete Boc[i];
    delete Boccn[i];
    delete Lno[i];
    delete Dlno[i];
    delete [] Nlsv[i];
    delete [] Nccn[i];
    delete Inic[i];
  }
  for (i = 0; i < nprope; i++)
  {
    delete Lel[i];
//    delete Dlel[i];
    delete [] Mel[i];
    delete [] Melid[i];
    delete [] Elsv[i];
    delete Edgl[i];
    delete Eps[i];
  }
  delete [] Nnp;
  delete [] Nep;
  delete [] Ndofn;
  delete [] Boc;
  delete [] Boccn;
  delete [] Lno;
  delete [] Dlno;
  delete [] Lel;
//  delete [] Dlel;
  delete [] Tel;
  delete [] Sel;
  delete [] Mel;
  delete [] Melid;
  delete [] Nmel;
  delete [] Ncs;
  delete [] Ncsi;
  delete [] Ecs;
  delete [] Ecsi;
  delete [] Nlsv;
  delete [] Elsv;
  delete [] Nccn;
  delete [] Edgl;
  delete Dbmat;
  delete Dbcrs;
  delete [] Tndofn;
  delete [] Tet;
  delete [] Nmu;
  delete [] Ncu;
  delete [] Inic;
  delete [] Enip;
  delete [] Edgl;
  delete [] Eep;
  delete [] Eps;
  delete Top;
  return (0);
}

long wr_nodes(FILE *out, long nprop)
/**<
 Function writes section with description of nodes to the text file given by out.

 @param out - pointer to the opened text file, where the data will be written
 @param nprop - number of entries in the input property file

 @retval 0 - if succes
 @retval 1 - if node hasn't assigned ndof
 @retval 2 - if node hasn't assigned cross-section
*/
{
  long i, j, k, l, *ic, ndf, ncf, nlsf, rindex;

  Tndofn =  new long[Top->nodes];
  memset(Tndofn, 0, sizeof(*Tndofn)*Top->nodes);
  ic = new long[Top->nodes];

  fprintf(out, "%ld\n", Top->nodes);
  for (i = 0; i < Top->nodes; i++)
  {
    for (j = 0; j < nprop; j++)
    // loop over all properties from input
    {
      if (Top->N[i].property == Nnp[j])
      // test if number of property matchs number from input
      {
        if (Ncs[j])
        // test if any cross-section is assigned
        {
          if ((ic[i] = Dbcrs->search_crs(Ncs[j], Ncsi[j]-1)) < 0) // searching cs in the cs database
          {
            fprintf(stderr,
                "\nError - cross-section with number %d and index %ld in node %ld not found\n", Ncs[j], Ncsi[j], i+1);
            delete [] ic;
            return(2);
          }
          else
          {
            if (Dbcrs->crs[ic[i]].propu[Ncsi[j]-1] == 0)
              Dbcrs->crs[ic[i]].npropu++; // indicator whether this cs type is used
            if (Ncu[ic[i]] == 0)
              Ncu[ic[i]]++;   // indicator whether cs type from cs database is used in the given task
            Dbcrs->crs[ic[i]].propu[Ncsi[j]-1]++; // indicator whether this index of cs is used
          }
          break;
        }
      }
    }
/*    if (ncf == 0)
    {
      fprintf(stderr, "\nError - node number %ld hasn't assigned cross-section\n", i+1);
      delete [] ic;
      return (2);
    }*/
  }
  Csrin = 0;
  for (j = 0; j < Dbcrs->numt; j++)
  {
    if (Dbcrs->crs[j].npropu)
    {
      rindex = 1;
      for (k = 0; k < Dbcrs->crs[j].nprop; k++)
      {
        if (Dbcrs->crs[j].propu[k])
        {
          Dbcrs->crs[j].ridx[k] = rindex;
          rindex++;
        }
      }
    }
  }
  Csrin = rindex;
  for (i = 0; i < Top->nodes; i++)
  {
    // writing node number and coordinates
    fprintf(out, "%6ld % e % e % e", i+1, Top->N[i].x, Top->N[i].y, Top->N[i].z);
    for (j = 0, ndf = 0; j < nprop; j++)
    // loop over all properties from input
    {
      if (Top->N[i].property == Nnp[j])
      // test if number of property matchs number from input
      {
        if (Ndofn[j])
        // test if node has assigned ndofs
        {
          fprintf(out, " %ld", Ndofn[j]);
          Tndofn[i] = Ndofn[j];
          ndf = 1;
          break;
        }
      }
    }
    if (ndf == 0)
    {
      fprintf(stderr, "\nError - node number %ld hasn't assigned ndofs\n", i+1);
      delete [] ic;
      return (1);
    }
    for (j = 0, ncf = 0; j < nprop; j++)
    // loop over all properties from input
    {
      if (Top->N[i].property == Nnp[j])
      // test if number of property matchs number from input
      {
        if (Ncs[j])
        // test if any cross-section is assigned
        {
          fprintf(out, " %d %ld", Ncs[j], Dbcrs->crs[ic[i]].ridx[Ncsi[j]-1]);
          ncf = 1;
          break;
        }
      }
    }
    if (ncf == 0)
    // no cross section assigned
      fprintf(out, " 0");
    for (j = 0, nlsf = 0; j < nprop; j++)
    // loop over all properties from input
    {
      if (Top->N[i].property == Nnp[j])
      // test if number of property matchs number from input
      {
        if (Nlsv[j])
        // test if any local system is assigned
        {
          nlsf = 1;
          fprintf(out, " %ld", Nlsv[j][0].n);
          for (k = 0; k < Nlsv[j][0].n; k++)
          {
            for (l = 0; l < Nlsv[j][k].n; l++)
              fprintf(out, " % e", Nlsv[j][k](l));
          }
          fprintf(out, "\n");
          break;
        }
      }
    }
    if (nlsf == 0)
      fprintf(out, " 0\n");
  }
  fprintf(out, "\n");
  delete [] ic;
  return(0);
}



/**
 Function writes section with description of boundary conditions of nodes
  to the text file given by out.

 @param out - pointer to the opened text file, where the data will be written
 @param nprop - number of entries in the input property file

 @retval 0 - if succes
 @retval 1 - if nodes haven't assigned any boundary condition
*/
long wr_bocon(FILE *out, long nprop)
{
  long i, j, k, nbc, tcn;

  nbc = 0;
  tcn = 0;
  Nspd = 0;
  Ndpd = 0;
  Boccn = new long *[nprop];
  memset(Boccn, 0, nprop*sizeof(*Boccn));
  for (i = 0; i < Top->nodes; i++)
  {
    for (j = 0; j < nprop; j++)
    // loop over all properties from input
    {
      if (Top->N[i].property == Nnp[j])
      // test if number of property matchs number from input
      {
        if (Boc[j])
        // test if any boundary condition is prescribed
        {
          nbc++;
          if (Boccn[j] == NULL)
          {
            Boccn[j] = new long[Tndofn[i]];
            memset(Boccn[j], 0, Tndofn[i] * sizeof(long));
            for (k = 0; k < Tndofn[i]; k++)
            // loop for assigning bc code numbers
            // if direction has bc, increase cn
            {
              if ((Boc[j]->dir[k] >= 0) && (Boc[j]->con[k] != 0.0))
              {
                tcn++;
                Boccn[j][k] = tcn;
                if (Boc[j]->expr[k] == NULL)
                  Nspd++;
                else
                  Ndpd++;
              }
            }
          }
        }
        if (Nccn[j])
        // test if any common code numbers are prescribed
          nbc++;

      }
    }
  }
/*  if (nbc == 0)
  {
    fprintf(stderr, "\nError - problem hasn't assigned any boundary condition\n");
    return(1);
  }*/
  fprintf(out, "%ld\n", nbc);
  for (i = 0; i < Top->nodes; i++)
  {
    for (j = 0; j < nprop; j++)
    // loop over all properties from input
    {
      if (Top->N[i].property == Nnp[j])
      // test if number of property matchs number from input
      {
        if ((Boc[j]) || (Nccn[j]))
        // test if any boundary condition is prescribed or any common code numbers are prescribed
        {
          fprintf(out, "%ld", i+1);
          for (k = 0; k < Tndofn[i]; k++)
          // loop for writing bc
          // if direction has bc, write 0 else write 1
          {
            if (Boc[j] && Nccn[j])
            {
              if ((Boc[j]->dir[k] > 0) && (Nccn[j][k]))
              {
                fprintf(stderr, "\n\nError - node %ld has prescribed boundary condition and common code number\n", i+1);
                fprintf(stderr, " in the same direction\n.");
                fprintf(stderr, " function wr_bocon (input.cpp)\n.");
                return(2);
              }
              if ((Boc[j]->dir[k] < 0) && (Nccn[j][k] == 0))
              // no boundary condition nor common code number
              {
                fprintf(out, " 1");
                continue;
              }
            }
            if (Boc[j])
            {
              if (Boc[j]->dir[k] < 0)
              // no boundary condition
              {
                fprintf(out, " 1");
                continue;
              }
              if (Boc[j]->con[k] == 0.0)
              // rigid support
              {
                fprintf(out, " 0");
                continue;
              }
              if (Boc[j]->con[k] != 0.0)
              // prescribed displacement
              {
                fprintf(out, " %ld", -Boccn[j][k]);
                continue;
              }
            }
            if (Nccn[j])
            // common code number
            {
              if (Nccn[j][k])
              {
                fprintf(out, " %ld", Nccn[j][k]);
                continue;
              }
              if (Nccn[j][k] == 0)
              {
                fprintf(out, " 1");
                continue;
              }
            }
          }
          fprintf(out, "\n");
          break;
        }
      }
    }
  }
  fprintf(out, "\n");
  return(0);
}



/**
 Function writes section with description of elements to the text file given by out.
  @param out - pointer to the opened text file, where the data will be written
  @param nprop - number of entries in the input property file

  @retval 0 : on succes
  @retval 1 : if element hasn't assigned type
*/
long wr_elements(FILE *out, long nprop, descrip *d)
{
  long i, j, k, tip;
  crsectype *ce;
  long *ci, *ic;
  strastrestate ssst;
  long rindex;
  elemtype et;

  mattype **me;
  long **mi, **im, *nm;

  me   = new mattype* [Top->elements];
  memset(me, 0, sizeof(*me)*Top->elements);
  mi   = new long* [Top->elements];
  memset(mi, 0, sizeof(*mi)*Top->elements);
  im   = new long* [Top->elements];
  memset(im, 0, sizeof(*im)*Top->elements);
  nm   = new long  [Top->elements];

  Tet = new elemtype[Top->elements];
  memset(Tet, 0, sizeof(*Tet)*Top->elements);
  ce   = new crsectype [Top->elements];
  memset(ce, 0, sizeof(*ce)*Top->elements);
  ci   = new long  [Top->elements];
  memset(ci, 0, sizeof(*ci)*Top->elements);
  ic   = new long  [Top->elements];

  fprintf(out, "%ld\n", Top->elements);
  for (i = 0; i < Top->elements; i++)
  {
    for (j = 0, et = elemtype(-1); j < nprop; j++)
    // loop over all properties from input
    {
      // test if number of property matchs number from input
      if ((Top->E[i].property == Nep[j]) && (Sel[j] == Top->E[i].type))
      {
        if (Tel[j] >= 0)
        // assigning element type
        {
          et = Tel[j];
          ssst = Ssse[j];
          break;
        }
      }
    }
    if (et < 0)
    {
      fprintf(stderr, "\nError - element number %ld hasn't assigned type\n", i+1);
      for (k = 0; k < Top->elements; k++)
      {
        delete [] me[k]; delete [] mi[k]; delete [] im[k];
      }
      delete [] me; delete [] mi; delete [] nm;
      delete [] ce; delete [] ci; delete [] ic;
      return(1);
    }
    for (j = 0; j < nprop; j++)
    // loop over all properties from input
    {
      if (Ecs[j])
      // assigning cross-section to the element with number i
      {
        ce[i] = Ecs[j];
        ci[i] = Ecsi[j];
      }
    }
    if (ce[i] > 0)
    {
      if ((ic[i] = Dbcrs->search_crs(ce[i], ci[i]-1)) < 0)
      {
        fprintf(stderr, "\nError - cross-section with number %d and index %ld not found\n", ce[i], ci[i]);
        delete [] ce; delete [] ci; delete [] ic;
        return(1);
      }
      else
      {
        if (Dbcrs->crs[ic[i]].propu[ci[i]-1] == 0)
          Dbcrs->crs[ic[i]].npropu++; // indicator whether given cs type is used
        if (Ncu[ic[i]] == 0)
          Ncu[ic[i]]++; // indicator whether cs type from cs database is used in the given task
        Dbcrs->crs[ic[i]].propu[ci[i]-1]++;  // indicator whether given cs index is used
      }
    }
    for (j = 0; j < Dbcrs->numt; j++)
    {
      if (Dbcrs->crs[j].npropu)
      {
        rindex = 1;
        for (k = 0; k < Dbcrs->crs[j].nprop; k++)
        {
          if (Dbcrs->crs[j].propu[k])
          {
            if (Dbcrs->crs[j].ridx[k] != 0)
              rindex++;
          }
        }
        for (k = 0; k < Dbcrs->crs[j].nprop; k++)
        {
          if (Dbcrs->crs[j].propu[k])
          {
            if (Dbcrs->crs[j].ridx[k] == 0)
            {
              Dbcrs->crs[j].ridx[k] = rindex;
              rindex++;
            }
          }
        }
      }
    }
    nm[i] = 0;
    for (j = 0; j < nprop; j++)
    // loop over all properties from input
    {
      // test if number of property matchs number from input
      if (Top->E[i].property == Nep[j])
      {
        if (Mel[j])
        // assigning matrial to the element with number i
        {
          nm[i] = Nmel[j];
          me[i] = new mattype[Nmel[j]];
          mi[i] = new long[Nmel[j]];
          im[i] = new long[Nmel[j]];
          for (k = 0; k < Nmel[j]; k++)
          {
            me[i][k] = Mel[j][k];
            mi[i][k] = Melid[j][k];
          }
        }
      }
    }
    if (me[i] ==  NULL) // unassigned element material
    {
      fprintf(stderr, "\nError - element %ld hasn't assigned material\n", i+1);
      for (k = 0; k < Top->elements; k++)
      {
        delete [] me[k]; delete [] mi[k]; delete [] im[k];
      }
      delete [] me; delete [] mi; delete [] nm;
      return(1);
    }
    for (j = 0; j < nm[i]; j++)
    {
      // searching element material in the material database
      if ((im[i][j] = Dbmat->search_mat(me[i][j], mi[i][j]-1)) < 0)
      {
        fprintf(stderr,
                "\nError - material with number %d and index %ld not found\n", me[i][j], mi[i][j]);
        for (k = 0; k < Top->elements; k++)
        {
          delete [] me[k]; delete [] mi[k]; delete [] im[k];
        }
        delete [] me; delete [] mi; delete [] nm;
        return(1);
      }
      else
      {
        if (Dbmat->mat[im[i][j]].propu[mi[i][j]-1] == 0)
          Dbmat->mat[im[i][j]].npropu++; // indicator whether given material type is used
        if (Nmu[im[i][j]] == 0)
          Nmu[im[i][j]]++;  // indicator whether material type from material database is used in the given task
        Dbmat->mat[im[i][j]].propu[mi[i][j]-1]++; // indicator whether given material index is used
      }
    }
    for (j = 0; j < Dbmat->numt; j++)
    {
      if (Dbmat->mat[j].npropu)
      {
        rindex = 1;
        for (k = 0; k < Dbmat->mat[j].nprop; k++)
        {
          if (Dbmat->mat[j].propu[k])
          {
            Dbmat->mat[j].ridx[k] = rindex;
            rindex++;
          }
        }
      }
    }
    tip = 0;
    switch (et)  // determining number of integration points on the element and
                 //  whether ip needs cross-section
    {
      case beam2d:
      {
        tip = 1;
        break;
      }
      default :
        break;
    }
    if ((tip) && (ce[i] == 0))
    {
      fprintf(stderr, "\nError - element %ld hasn't assigned cross-section\n", i+1);
      delete [] ce; delete [] ci; delete [] ic;
      return(3);
    }
    Mt->elements[i].te = et;
    // writing type of element, element number
    fprintf(out, "%6ld ", i+1);
    if ((et >= planeelementlt) && (et <= planeelementrotlq))
      fprintf(out, "%3d %3d", et, ssst);
    else
      fprintf(out, "%3d", et);
    Tet[i] = et;
    // writing element nodes
    for (j = 0; j < Top->E[i].size; j++)
    {
/*      if ((et == planeelementqt) && (d->t3d) && (j == 3))
      // nodes in T3d file has in this case folowing order 1 2 3 5 6 4 in view of
      // SIFEL order 1 2 3 4 5 6
        continue;*/
      fprintf(out, " %6ld", Top->E[i].node[j]);
    }
/*    if ((et == planeelementqt) && (d->t3d))
    // nodes in T3d file has in this case folowing order 1 2 3 5 6 4 in view of
    // SIFEL order 1 2 3 4 5 6
    // so it is need to write fourth node
      fprintf(out, " %6ld", Top->E[i].node[3]);*/

    // no code numbers on element (code numbers not supported at this time)
    fprintf(out, " 0");
    if (ce[i] > 0)
      fprintf(out, " %d %ld", ce[i], Dbcrs->crs[ic[i]].ridx[ci[i]-1]); // if element has cs, write cs type and index
    else
      fprintf(out, " 0"); // writing element cross-section indicator
    for (l = 0; l < nm[i]; l++)
      fprintf(out, "%d %ld ", me[i][l], Dbmat->mat[im[i][l]].ridx[mi[i][l]-1]); // writing element material type and index
    fprintf(out, "\n");
  }
  fprintf(out, "\n");
  return(0);
}



/**<
 Function writes section with description of elements to the text file given by out.

 @param out - pointer to the opened text file, where the data will be written

 @retval 0 : on succes
*/
long wr_globnodnum(FILE *out)
{
  for (long i = 0; i < Top->nodes; i++)
    fprintf(out, "%ld\n", Top->Pn[i]);
  return(0);
}

/**
 Function writes section with description of elements to the text file given by out.

  @param out - pointer to the opened text file, where the data will be written
  @param nprop - number of entries in the input property file

  @retval 0 : if succes
  @retval 1 : if element hasn't assigned material
  @retval 2 : if element hasn't assigned type
  @retval 3 : if element hasn't assigned cross-section
*/
long wr_intpoints(FILE *out, long nprop)
{
  long i, j, k, l;
  mattype **me;
  long **mi, **im, *nm;
  elemtype et = elemtype(-1);
  long rindex;

  me   = new mattype* [Top->elements];
  memset(me, 0, sizeof(*me)*Top->elements);
  mi   = new long* [Top->elements];
  memset(mi, 0, sizeof(*mi)*Top->elements);
  im   = new long* [Top->elements];
  memset(im, 0, sizeof(*im)*Top->elements);
  nm   = new long  [Top->elements];
  Enip = new long  [Top->elements];
  memset(Enip, 0, sizeof(*Enip)*Top->elements);

//  Nmu = 0;
  for (i = 0; i < Top->elements; i++)
  {
    nm[i] = 0;
    for (j = 0; j < nprop; j++)
    // loop over all properties from input
    {
      // test if number of property matchs number from input
      if (Top->E[i].property == Nep[j])
      {
        if (Mel[j])
        // assigning matrial to the element with number i
        {
          nm[i] = Nmel[j];
          me[i] = new mattype[Nmel[j]];
          mi[i] = new long[Nmel[j]];
          im[i] = new long[Nmel[j]];
          for (k = 0; k < Nmel[j]; k++)
          {
            me[i][k] = Mel[j][k];
            mi[i][k] = Melid[j][k];
          }
        }
      }
    }
    et = Tet[i];
    if (et < 0) // unassigned element type
    {
      fprintf(stderr, "\nError - element %ld hasn't assigned type\n", i+1);
      for (k = 0; k < Top->elements; k++)
      {
        delete [] me[k]; delete [] mi[k]; delete [] im[k];
      }
      delete [] me; delete [] mi; delete [] nm;
      return(1);
    }
    if (me[i] ==  NULL) // unassigned element material
    {
      fprintf(stderr, "\nError - element %ld hasn't assigned material\n", i+1);
      for (k = 0; k < Top->elements; k++)
      {
        delete [] me[k]; delete [] mi[k]; delete [] im[k];
      }
      delete [] me; delete [] mi; delete [] nm;
      return(1);
    }


    for (j = 0; j < nm[i]; j++)
    {
      // searching element material in the material database
      if ((im[i][j] = Dbmat->search_mat(me[i][j], mi[i][j]-1)) < 0)
      {
        fprintf(stderr,
                "\nError - material with number %d and index %ld not found\n", me[i][j], mi[i][j]);
        for (k = 0; k < Top->elements; k++)
        {
          delete [] me[k]; delete [] mi[k]; delete [] im[k];
        }
        delete [] me; delete [] mi; delete [] nm;
        return(1);
      }
      else
      {
        if (Dbmat->mat[im[i][j]].propu[mi[i][j]-1] == 0)
          Dbmat->mat[im[i][j]].npropu++; // indicator whether given material type is used
        if (Nmu[im[i][j]] == 0)
          Nmu[im[i][j]]++;  // indicator whether material type from material database is used in the given task
        Dbmat->mat[im[i][j]].propu[mi[i][j]-1]++; // indicator whether given material index is used
      }
    }
    Enip[i] = Mt->get_nip(i,0);
  }
  for (j = 0; j < Dbmat->numt; j++)
  {
    if (Dbmat->mat[j].npropu)
    {
      rindex = 1;
      for (k = 0; k < Dbmat->mat[j].nprop; k++)
      {
        if (Dbmat->mat[j].propu[k])
        {
          Dbmat->mat[j].ridx[k] = rindex;
          rindex++;
        }
      }
    }
  }
  for (i = 0; i < Top->elements; i++)
  {
    for (k = 0; k < Enip[i]; k++)
    {
      for (l = 0; l < nm[i]; l++)
        fprintf(out, "%d %ld ", me[i][l], Dbmat->mat[im[i][l]].ridx[mi[i][l]-1]); // writing element material type and index
      fprintf(out, "\n");
    }
    fprintf(out, "\n");
  }
  fprintf(out, "\n");
  for (k = 0; k < Top->elements; k++)
  {
    delete [] me[k];
    delete [] mi[k];
    delete [] im[k];
  }
  delete [] me; delete [] mi; delete [] nm;
  return(0);
}



/**
 Function writes materials and thier parameters to the file given by the out.

  @param out - pointer to the opened text file, where the data will be written

  @retval 0 : on succes
*/
long wr_materials(FILE *out)
{
  long i, j, nmtu, wrt;
  for (i = 0, nmtu = 0; i < Dbmat->numt; i++)
  // countig number of used material types
  {
    if (Nmu[i])
      nmtu++;
  }
  fprintf(out, "%ld\n", nmtu);  // writing number of material types
  for (i = 0; i < Dbmat->numt; i++)
  {
    wrt = 1;
    for (j = 0; j < Dbmat->mat[i].nprop; j++)
    {
      if (Dbmat->mat[i].propu[j])  // material type index is used (<> 0)
      {
        if (wrt)  // for the first time write material type number and number of used indeces
        {
          fprintf(out, "%d %ld\n", Dbmat->mat[i].type, Dbmat->mat[i].npropu);
          wrt = 0;  // next time don't write
        }
        // write material type index and property string
        fprintf(out, "%ld %s\n", Dbmat->mat[i].ridx[j], Dbmat->mat[i].prop[j]);
      }
    }
  }
  fprintf(out, "\n");
  return(0);
}



long wr_auxpoint(FILE *out, long nprop)
{
  long i, j, psn = 0;
  for (i = 0; i < Top->elements; i++)
  {
    for (j = 0; j < nprop; j++)
    {
      if ((Top->E[i].property == Nep[j]) && (Sel[j] == Top->E[i].type))
      {
        if (Eep[j] >= 0)
        {
          fprintf(out, "%d", Eep[j]);
          if (Eps[j])
          {
            psn++;
            Eps[j]->n = psn;
            fprintf(out, " %ld", psn);
          }
          fprintf(out,"\n");
          break;
        }
      }
    }
  }
  if (psn != 0)
    fprintf(out, "%ld\n", psn);
  for (j = 0; j < nprop; j++)
  {
    if (Eps[j])
      Eps[j]->print(out);
  }
  fprintf(out, "\n");
  return(0);
}



/**
 Function writes section with local coordinate systems in points on elements
 to the text file given by out.

  @param out - pointer to the opened text file, where the data will be written
  @param nprop - number of entries in the input property file

  @retval 0 : if succes
*/
long wr_ellcsys(FILE *out, long nprop)
{
  long i, j, k;
  long *elsvi = NULL, telsvi = 0, nells = 0;
  long ips;
  pointset    *tps;
  elemposition tep;

  elsvi = new long[nprop];
  memset(elsvi, 0, sizeof(*elsvi)*nprop);


  // countig number of elements with l.c. system
  for (i = 0; i < Top->elements; i++)
  {
    for (j = 0; j < nprop; j++)
    {
      if (Top->E[i].property == Nep[j])
      {
        if (Elsv[j])
        {
          nells++;
          break;
        }
      }
    }
  }
  // indexing l. c. system
  for (i = 0; i < nprop; i++)
  {
    if (Elsv[i])
    {
      telsvi++;
      elsvi[i] = telsvi;
    }
  }

  fprintf(out, "%ld\n", nells);

  for (i = 0; i < Top->elements; i++)
  {
    ips = -1;
    // searching type of auxiliary points on element
    for (j = 0; j < nprop; j++)
    {
      if ((Top->E[i].property == Nep[j]) && (Sel[j] == Top->E[i].type))
      {
        if (Eep[j])
        {
          tep = Eep[j];
          tps = Eps[j];
          ips = j;
          break;
        }
      }
    }
    if (ips < 0)
    {
      fprintf(stderr, "\n\nError - element hasn't assigned position of auxiliary points");
      fprintf(stderr, "\nfunction wr_ellcsys() (output.cpp)\n");
      return(1);
    }
    for (j = 0; j < nprop; j++)
    {
      if (Top->E[i].property == Nep[j])
      {
        if (Elsv[j])
        {
          switch (tep)
          {
            case nowhere :
            {
              fprintf(out, "%ld %ld\n", i+1, 0L);
              break;
            }
            case intpts :
            {
              fprintf(out, "%ld %ld ", i+1, Enip[i]);
              for (k = 0; k < Enip[i]; k++)
                fprintf(out, " %ld %ld", k+1, elsvi[j]);
              fprintf(out, "\n");
              break;
            }
            case enodes :
            {
              fprintf(out, "%ld %ld", i+1, Top->E[i].size);
              for (k = 0; k < Top->E[i].size; k++)
                fprintf(out, " %ld %ld", k+1, elsvi[j]);
              fprintf(out, "\n");
              break;
            }
            case userdefined :
            {
              fprintf(out, "%ld %ld", i+1, Eps[j]->np);
              for (k = 0; k < Eps[j]->np; k++)
                fprintf(out, " %ld %ld", k+1, elsvi[j]);
              fprintf(out, "\n");
              break;
            }
            default :
            {
              fprintf(stderr, "\n\nError - unknown type of position of auxiliary points");
              fprintf(stderr, "\nfunction wr_ellcsys() (output.cpp)\n");
              break;
            }
          }
        }
      }
    }
  }
  // print base vectors of local coordinate system
  fprintf (out, "%ld\n", telsvi);
  for (i = 0; i < telsvi; i++)
  {
    fprintf(out, "%ld", Elsv[i][0].n*Elsv[i][0].n);
    for (j = 0; j < Elsv[i][0].n; j++)
    {
      for (k = 0; k < Elsv[i][0].n; k++)
        fprintf (out, " %e", Elsv[i][j][k]);
    }
    fprintf(out, "\n");
  }
  fprintf(out, "\n");
  delete [] elsvi;
  return(0);
}

/**
 Function writes cross-sections and thier parameters to the file given by the out.

  @param out - pointer to the opened text file, where the data will be written

  @retval 0 : on succes
*/
long wr_crsecs(FILE *out)
{
  long i, j, wrt, nctu;

//  fprintf(out, "%ld", Ecsn+Ncsn);
  for(i = 0, nctu = 0; i < Dbcrs->numt; i++)
  // countig number of used cross-section types
  {
    if (Ncu[i])
      nctu++;
  }
  fprintf(out, "%ld\n", nctu);    // writing number of cs types
  for (i = 0; i < Dbcrs->numt; i++)
  {
    wrt = 1;
    for (j = 0; j < Dbcrs->crs[i].nprop; j++)
    {
      if (Dbcrs->crs[i].propu[j])  // cs type index is used (<> 0)
      {
        if (wrt) // for the first time write cs type number and number of used indeces
        {
          fprintf(out, "%d %ld\n", Dbcrs->crs[i].type, Dbcrs->crs[i].npropu);
          wrt = 0;  // next time don't write
        }
        // write cs type index and property string
        fprintf(out, "%ld %s\n", Dbcrs->crs[i].ridx[j], Dbcrs->crs[i].prop[j]);
      }
    }
  }
  fprintf(out, "\n");
  return(0);
}



/**
 Function writes section with description of load to the text file given by out

  @param out - pointer to the opened text file, where the data will be written
  @param npropn - number of entries in the nodal section of property file
  @param nprope - number of entries in the element section of property file

  @retval 0 : on success
*/
long wr_load(FILE *out, long npropn, long nprope)
{
  long tnlc;
  if ((Mp->tprob == nonlinear_statics) ||
      (Mp->tprob == forced_dynamics))
  {
    fprintf(out, "%ld\n\n\n", Nlc/2);  // writing number of load cases
    tnlc = Nlc/2;
  }
  else
  {
    fprintf(out, "%ld\n\n\n", Nlc);  // writing number of load cases
    tnlc = Nlc;
  }
  for (long i = 0; i < tnlc; i++)
  {
    wr_loadn(out, npropn, i);     // writing nodal load
    wr_loadel(out, nprope, npropn, i);    // writing element load
    wr_prescdisp(out, npropn, i); // writing prescribed displacements
    fprintf(out, "\n");
    if (Mp->tprob == forced_dynamics)
    {
      wr_dloadn(out, npropn, i+1);     // writing dynamic nodal load
      wr_dloadel(out, nprope, i+1);    // writing dynamic element load
      wr_dprescdisp(out, npropn, i+1); // writing dynamic prescribed displacements
      wr_initcond(out, npropn, i+1);   // writing initial conditions
      fprintf(out, "\n");
      fflush(out);
    }
  }
  return(0);
}



/*
 Function writes section with description of dynamic load to the text file given by out

  @param out - pointer to the opened text file, where the data will be written
  @param npropn - number of entries in the nodal section of property file
  @param nprope - number of entries in the element section of property file

  @retval 0 : on success
*/
/*
long wr_dload(FILE *out, long npropn, long nprope)
{
  for (long i = 0; i < Nlc; i++)
  {
    wr_dloadn(out, npropn, i);     // writing dynamic nodal load
    wr_dloadel(out, nprope, i);    // writing dynamic element load
    wr_dprescdisp(out, npropn, i); // writing dynamic prescribed displacements
    if (Mp->tprob == forced_dynamics)
      wr_initcond(out, npropn, i); // writing initial conditions
    fprintf(out, "\n");
  }
  return(0);
}
*/


/**
 Function writes section with description of nodal load to the text file given by out

  @param out   - pointer to the opened text file, where the data will be written
  @param nprop - number of entries in the input property file
  @param nlc   - number of loading case which will be written

  @retval 0 : on succes
*/
long wr_loadn(FILE *out, long nprop, long nlc)
{
  long i, j, k, nln;
  for (i = 0, nln = 0; i < Top->nodes; i++)
  // countig overall number of nodal loads
  {
    for (j = 0; j < nprop; j++)
    {
      if (Top->N[i].property == Nnp[j])
      {
        if ((Lno[j]) && (Lno[j]->nlc == nlc+1))
          nln++;
      }
    }
  }
  fprintf(out, "%ld\n", nln);  // writing overall number of nodal loading entries
  for (i = 0; i < Top->nodes; i++)
  {
    for (j = 0; j < nprop; j++)
    {
      if (Top->N[i].property == Nnp[j])
      {
        if ((Lno[j]) && (Lno[j]->nlc == nlc+1))
        {
          // writing node number
          fprintf(out, "%6ld", i+1);
          for (k = 0; k < Tndofn[i]; k++)
          // loop for writing values of load in single directions
            fprintf(out, " %e", Lno[j]->f[k]);
          fprintf(out, "\n");
        }
      }
    }
  }
  fprintf(out, "\n");
  return(0);
}



/**
 Function writes section with description of dynamic nodal load to the text file given by out

  @param out   - pointer to the opened text file, where the data will be written
  @param nprop - number of entries in the input property file
  @param nlc   - number of loading case which will be written

  @retval 0 : on succes
*/
long wr_dloadn(FILE *out, long nprop, long nlc)
{
  long i, j, k, nln;
  for (i = 0, nln = 0; i < Top->nodes; i++)
  // countig overall number of nodal loads
  {
    for (j = 0; j < nprop; j++)
    {
      if (Top->N[i].property == Nnp[j])
      {
        if ((Dlno[j]) && (Dlno[j]->nlc == nlc+1))
          nln++;
      }
    }
  }
  fprintf(out, "%ld\n", nln);  // writing overall number of nodal loading entries
  for (i = 0; i < Top->nodes; i++)
  {
    for (j = 0; j < nprop; j++)
    {
      if (Top->N[i].property == Nnp[j])
      {
        if ((Dlno[j]) && (Dlno[j]->nlc == nlc+1))
        {
          // writing node number
          fprintf(out, "%6ld 1", i+1);
          for (k = 0; k < Tndofn[i]; k++)
          // loop for writing values of load in single directions
          {
            fprintf(out, "%6c %e", ' ', Dlno[j]->f[k]);
            fprintf(out, "%6c %s\n", ' ', Dlno[j]->func[k]);
          }
          fprintf(out, "\n");
        }
      }
    }
  }
  fprintf(out, "\n");
  return(0);
}



/**
 Function writes section with description of element load to the text file given by out

  @param out   - pointer to the opened text file, where the data will be written
  @param nprop - number of entries in the input property file
  @param nlc   - number of loading case which will be written

  @retval 0 : on succes
*/
long wr_loadel(FILE *out, long nprope, long npropn, long nlc)
{
  long i, j, k, l, nle;
  for (i = 0, nle = 0; i < Top->elements; i++)
  // countig overall number of element loads
  {
    for (j = 0; j < nprope; j++)
    {
      if (Top->E[i].property == Nep[j])
      {
        if (Lel[j])
        {
          if(Lel[j]->nlc == nlc+1)
            nle++;
        }
      }
    }
  }

  nle += search_loadedg_el(nprope, nlc);
  // writing overall number of element loads entries
  fprintf(out, "%ld\n", nle);
  for (i = 0; i < Top->elements; i++)
  {
    for (j = 0; j < nprope; j++)
    {
      if (Top->E[i].property == Nep[j])
      {
        if (Lel[j])
        {
          if (Lel[j]->nlc == nlc+1)
          {
            // writing element number
            fprintf(out, "%6ld", i+1);
            // writing load type
            fprintf(out, " %d", (int)volume);
            for (k = 0; k < Top->E[i].size; k++)
            {
              for (l = 0; l < Lel[j]->ndl; l++)
              // loop for writing values of load in single directions
                fprintf(out, " %e", Lel[j]->nodval[k*Lel[j]->ndl+l]);
            }
            fprintf(out, "\n");
          }
        }
      }
    }
  }
  wr_loadeledg(out, nprope, nlc);
  fprintf(out, "\n");
  return(0);
}



long search_loadedg_el(long nprop, long nlc)
{
  long i, j, k;
  long nl = 0;
  long *elel;
  elel = new long[Top->elements];
  memset(elel, 0, sizeof(*elel)*Top->elements);
  for (i = 0; i < nprop; i++)
  {
    if (Edgl[i])
    {
      if (Edgl[i]->nlc == nlc+1)
      {
        for (j = 0; j < Top->elements; j++)
        {
          for (k = 0; k < Top->E[j].ned; k++)
          {
            if (Top->E[j].propedg[k] == Nep[i])
            {
              if (elel[j] == 0)
                nl++;
              elel[j] = 1;
              break;
            }
          }
        }
      }
    }
  }
  delete [] elel;
  return (nl);
}



long wr_loadeledg(FILE *out, long nprop, long nlc)
{
  long i, j, k, l, ll, id;
  long ned, napfun, nned;
  long *enod = NULL;
  double *nodvals = NULL;
  double *nv;
  snode *tn;

  for (i = 0; i < Top->elements; i++)
  {
    ned    = Mt->get_ned(i);
    nned   = Mt->get_nned(i);
    napfun = Mt->get_napfun(i);
    for (j = 0; j < nprop; j++)
    {
      if (Edgl[j])
      {
        if (Edgl[j]->nlc == nlc+1)
        {
          for (k = 0; k < ned; k++)
          {
            enod = new long[nned];
            if (Top->E[i].propedg[k] == Nep[j])
            {
              if (nodvals == NULL)
              {
                nodvals = new double [ned*nned*napfun];
                memset(nodvals, 0, sizeof(*nodvals)*ned*nned*napfun);
              }
              Mt->get_edge_nodes(i, k, enod);
              nv = new double[napfun];
              for (l = 0; l < nned; l++)
              {
                memset(nv, 0, sizeof(*nv)*napfun);
                tn = &Top->N[enod[l]-1];
                Edgl[j]->getval(tn, nv);
                id = k*nned*napfun + l*napfun;
                for (ll = 0; ll < napfun; ll++)
                  nodvals[id+ll] = nv[ll];
//                memcpy(nodvals+id, nv, sizeof(*nv)*napfun);
              }
              delete [] nv;
            }
            delete [] enod;
          }
        }
      }
    }
    if (nodvals)
    {
      id = 0;
      fprintf(out, "%ld", i+1);
      fprintf(out, " %d", (int)edge);
      for (j = 0; j < ned; j++)
      {
        fprintf(out, " 1");
        for (k = 0; k < nned; k++)
        {
          for (l = 0; l < napfun; l++)
          {
            fprintf(out, " %le", nodvals[id]);
            id++;
          }
        }
        fprintf(out, "\n");
      }
      delete [] nodvals;
      nodvals = NULL;
    }
  }
  fprintf(out, "\n");
  return (0);
}


long wr_dloadel(FILE *out, long nprop, long nlc)
/*
 NOT IMPLEMENTED YET
 Function writes section with description of dynamic element load to the text file given by out

  @param out   - pointer to the opened text file, where the data will be written
  @param nprop - number of entries in the input property file
  @param nlc   - number of loading case which will be written

  @retval 0 : on succes
*/
{
/*
  long i, j, k, l, nle;
  for (i = 0, nle = 0; i < Top->elements; i++)
  // countig overall number of element loads
  {
    for (j = 0; j < nprop; j++)
    {
      if (Top->E[i].property == Nep[j])
      {
        if ((Lel[j]) && (Lel[j]->nlc == nlc+1))
          nle++;
      }
    }
  }
  // writing overall number of element loads entries
  fprintf(out, "%ld\n", nle);
  for (i = 0; i < Top->elements; i++)
  {
    for (j = 0; j < nprop; j++)
    {
      if (Top->E[i].property == Nep[j])
      {
        if ((Lel[j]) && (Lel[j]->nlc == nlc+1))
        {
          // writing element number
          fprintf(out, "%6ld", i+1);
          for (k = 0; k < Top->E[i].size; k++)
          {
            for (l = 0; l < Lel[j]->ndl; l++)
            // loop for writing values of load in single directions
              fprintf(out, " %e", Lel[j]->f[k*Lel[j]->ndl+l]);
          }
          fprintf(out, "\n");
        }
      }
    }
  }
  fprintf(out, "\n");*/
  return(0);
}



/**
 Function writes section with description of nodal prescribed
  displacement load to the text file given by out.

  @param out   - pointer to the opened text file, where the data will be written
  @param nprop - number of entries in the input property file
  @param nlc   - number of loading case which will be written

  @retval 0 : on succes
*/
long wr_prescdisp(FILE *out, long nprop, long nlc)
{
  long i, j;

  fprintf(out, "%ld\n", Nspd);
  if (Nspd == 0)
    return(0);
  for (i = 0; i < nprop; i++)
  // loop over all properties from input
  {
    if (Boc[i])
    // test if any boundary condition is prescribed
    {
      for (j = 0; j < Boc[i]->ndir; j++)
      // loop for writing bc
      // if direction has bc, write p.d. number and value
      {
        if ((Boc[i]->dir[j] >= 0) && (Boc[i]->con[j] != 0.0))
          fprintf(out, " %ld %e\n", Boccn[i][j], Boc[i]->con[j]);
      }
    }
  }
  fprintf(out, "\n");
  return(0);
}



/**
 Function writes section with description of dynamic nodal prescribed
  displacement load to the text file given by out.

  @param out   - pointer to the opened text file, where the data will be written
  @param nprop - number of entries in the input property file
  @param nlc   - number of loading case which will be written

  @retval 0 : on succes
*/
long wr_dprescdisp(FILE *out, long nprop, long nlc)
{
  long i, j;


  fprintf(out, "%ld\n", Ndpd);
  if (Ndpd == 0)
    return(0);
  for (i = 0; i < nprop; i++)
  // loop over all properties from input
  {
    if (Boc[i])
    // test if any boundary condition is prescribed
    {
      for (j = 0; j < Boc[i]->ndir; j++)
      // loop for writing bc
      // if direction has bc, write p.d. number and value
      {
        if ((Boc[i]->dir[j] >= 0) && (Boc[i]->con[j] != 0.0) && (Boc[i]->expr[j] != NULL))
          fprintf(out, " %ld %e %s\n", Boccn[i][j], Boc[i]->con[j], Boc[i]->expr[j]);
      }
    }
  }
  fprintf(out, "\n");
  return(0);
}

/**
    Function writes section with description of dynamic nodal prescribed
     displacement load to the text file given by out.

    @param out   - pointer to the opened text file, where the data will be written
    @param nprop - number of entries in the input property file
    @param nlc   - number of loading case which will be written

    @retval 0 : on succes
*/
long wr_initcond(FILE *out, long nprop, long nlc)
{
  long i, j, k;
  for (i = 0; i < Top->nodes; i++)
  {
    for (j = 0; j < nprop; j++)
    {
      if (Top->N[i].property == Nnp[j])
      {
        if (Inic[j])
        {
          if (Inic[j]->nlc == nlc+1)
          {
            // writing node number
            fprintf(out, "%6ld ", i+1);
            for (k = 0; k < Tndofn[i]; k++)
            // loop for writing values of initial conditions in single directions
              fprintf(out, "%e ", Inic[j]->con[k]);
            fprintf(out, "\n");
          }
        }
      }
    }
  }
  fprintf(out, "\n");
  return(0);
}
