#include <math.h>
#include "barelq2d.h"
#include "barel2d.h"
#include "global.h"
#include "globmat.h"
#include "genfile.h"
#include "element.h"
#include "node.h"
#include "intpoints.h"
#include "loadcase.h"



barelq2d::barelq2d (void)
{
  long i;

  nne=3;  ndofe=6;  tncomp=1;  napfun=2;
  tnip=3;
  ssst = bar;

  nb=1;

  ncomp = new long [nb];
  ncomp[0]=1;

  cncomp = new long [nb];
  cncomp[0]=0;

  nip = new long* [nb];
  intordsm = new long* [nb];
  for (i=0;i<nb;i++){
    nip[i] = new long [nb];
    intordsm[i] = new long [nb];
  }

  nip[0][0]=3;

  intordsm[0][0]=3;

  zero=Mp->zero;
}

barelq2d::~barelq2d (void)
{
  long i;

  for (i=0;i<nb;i++){
    delete [] nip[i];
    delete [] intordsm[i];
  }
  delete [] nip;
  delete [] intordsm;

  delete [] cncomp;
  delete [] ncomp;
  delete [] nip;
}

void barelq2d::eleminit (long eid)
{
  long ii,jj;
  Mt->elements[eid].nb=nb;
  Mt->elements[eid].intordsm = new long* [nb];
  Mt->elements[eid].nip = new long* [nb];

  for (ii=0;ii<nb;ii++){
    Mt->elements[eid].intordsm[ii] = new long [nb];
    Mt->elements[eid].nip[ii] = new long [nb];
    for (jj=0;jj<nb;jj++){
      Mt->elements[eid].intordsm[ii][jj]=intordsm[ii][jj];
      Mt->elements[eid].nip[ii][jj]=nip[ii][jj];
    }
  }
}


/**
   function computes direction %vector
   
   @param s - direction %vector
   @param x,y - vectors of nodal coordinates
   
   TKR podle JK, 31.10.2006
*/
void barelq2d::dirvect (vector &s,vector &x,vector &y)
{
  double d;
  
  s[0]=x[1]-x[0];
  s[1]=y[1]-y[0];
  
  d=sqrt(s[0]*s[0]+s[1]*s[1]);
  
  if (d<zero){
    fprintf (stderr,"\n\n zero norm of direction vector in function dirvect (file %s, line %d).\n",__FILE__,__LINE__);
  }
  
  s[0]/=d;
  s[1]/=d;
}



/**
   function approximates function defined by nodal values

   @param xi,eta - coordinates on element
   @param nodval - nodal values
*/
double barelq2d::approx (double xi,vector &nodval)
{
  double f;
  vector bf(nne);

  bf_quad_1d (bf.a,xi);

  scprd (bf,nodval,f);

  return f;
}

/**
   function returns geometric matrix

   @param gm - geometric matrix
   @param s,c - sin and cos
   @param l - length of the bar

   10.8.2001
*/
void barelq2d::geom_matrix (matrix &gm, vector &x,vector &s,double xi,double &jac)
{
  vector dx(nne);
  
  dx_bf_quad_1d (dx.a,xi);
  derivatives_1d (dx, jac, x, xi);
  
  fillm(0.0, gm);
  
  gm[0][0]= dx[0]*s[0];
  gm[0][1]= dx[0]*s[1];
  gm[0][2]= dx[1]*s[0];
  gm[0][3]= dx[1]*s[1];
  gm[0][4]= dx[2]*s[0];
  gm[0][5]= dx[2]*s[1];
}


void barelq2d::giveloccoord(vector &x, vector &y,vector &lx)
{
  double l;

  lx[0] = 0.0;
  l = sqr(x[1] - x[0]) + sqr(y[1] - y[0]);
  l = sqrt(l);
  lx[1] = l;
  l = sqr(x[2] - x[0]) + sqr(y[2] - y[0]);
  l = sqrt(l);
  lx[2] = l;
}

void barelq2d::give_glob_loc_tmat(vector &x, vector &y, matrix &tmat)
{
  double c, s, l;

  fillm(0.0, tmat);
  l = sqrt(sqr(x[1]-x[0]) + sqr(y[1]-y[0]));
  c = (x[1] - x[0]) / l;
  s = (y[1] - y[0]) / l;
  tmat[0][0] = tmat[2][2] = tmat[4][4] =  c;
  tmat[0][1] = tmat[2][3] = tmat[4][5] =  s;
  tmat[1][0] = tmat[3][2] = tmat[5][4] = -s;
  tmat[1][1] = tmat[3][3] = tmat[5][5] =  c;
}

void barelq2d::give_loc_glob_tmat(vector &x, vector &y, matrix &tmat)
{
  double c, s, l;

  fillm(0.0, tmat);
  l = sqrt(sqr(x[1]-x[0]) + sqr(y[1]-y[0]));
  c = (x[1] - x[0]) / l;
  s = (y[1] - y[0]) / l;
  tmat[0][0] = tmat[2][2] = tmat[4][4] =  c;
  tmat[0][1] = tmat[2][3] = tmat[4][5] = -s;
  tmat[1][0] = tmat[3][2] = tmat[5][4] =  s;
  tmat[1][1] = tmat[3][3] = tmat[5][5] =  c;
}

/**
   transformation matrix x_g = T x_l
*/
void barelq2d::transf_matrix (ivector &nodes,matrix &tmat)
{
  long i;
  fillm (0.0,tmat);

  for (i=0;i<tmat.m;i++){
    tmat[i][i]=1.0;
  }

  for (i=0;i<nodes.n;i++){
    if (Mt->nodes[nodes[i]].transf>0){
      tmat[i*2][i*2]=Mt->nodes[nodes[i]].e1[0];    tmat[i*2][i*2+1]=Mt->nodes[nodes[i]].e2[0];
      tmat[i*2+1][i*2]=Mt->nodes[nodes[i]].e1[1];  tmat[i*2+1][i*2+1]=Mt->nodes[nodes[i]].e2[1];
    }
  }

}


/**
   function computes stiffness matrix of one element

   @param eid - element id
   @param ri - row index
   @param sm - stiffness matrix

   10.8.2001
*/
void barelq2d::stiffness_matrix (long eid,long ri,long ci,matrix &sm,vector &x,vector &s)
{
  long i,ipp;
  double a,xi,jac;
  ivector nodes (nne);
  vector w,gp,t(nne);
  matrix gm,d(tncomp,tncomp);

  fillm (0.0,sm);

  Mt->give_elemnodes (eid,nodes);
  Mc->give_area (eid,a);

  allocv (intordsm[0][0],w);
  allocv (intordsm[0][0],gp);
  gauss_points (gp.a,w.a,intordsm[0][0]);
  
  allocm (ncomp[0],ndofe,gm);
  ipp=Mt->elements[eid].ipp[ri][ci];
  
  for (i=0;i<intordsm[0][0];i++){
    xi=gp[i];

    geom_matrix (gm, x, s, xi, jac);
    //  matrix of stiffness of the material
    Mm->matstiff (d,ipp);
    jac*=a*w[i];
    //  contribution to the stiffness matrix of the element
    bdbjac (sm,gm,d,gm,jac);

    ipp++;
  }
  
}

void barelq2d::mass_matrix (long eid,matrix &mm)
{
/*
  double l,a,rho;
  vector x(nne),y(nne);

  fillm (0.0,mm);

  Mt->give_node_coord2d (x,y,eid);

  //  length of the element
  l=sqrt ((x[1]-x[0])*(x[1]-x[0])+(y[1]-y[0])*(y[1]-y[0]));
  if (l<Mp->zero){
    fprintf (stderr,"\n\n zero length of the %ld truss element",eid);
    fprintf (stderr,"\n in function barel2d::mass_matrix (%s, line %d).\n",__FILE__,__LINE__);
  }
  Mc->give_area (eid,a);
  Mc->give_densitye (eid,rho);

  mm[0][0]=a*rho*l*1.0/3.0;
  mm[0][1]=0.0;
  mm[0][2]=a*rho*l*1.0/6.0;
  mm[0][3]=0.0;

  mm[1][0]=0.0;
  mm[1][1]=a*rho*l*1.0/3.0;
  mm[1][2]=0.0;
  mm[1][3]=a*rho*l*1.0/6.0;

  mm[2][0]=a*rho*l*1.0/6.0;
  mm[2][1]=0.0;
  mm[2][2]=a*rho*l*1.0/3.0;
  mm[2][3]=0.0;

  mm[3][0]=0.0;
  mm[3][1]=a*rho*l*1.0/6.0;
  mm[3][2]=0.0;
  mm[3][3]=a*rho*l*1.0/3.0;
*/
}











void barelq2d::res_mainip_strains (long lcid,long eid)
{
  vector aux,gx(nne),gy(nne),x(nne),s(2),r(ndofe);
  ivector cn(ndofe),nodes(nne);
  matrix tmat;

  Mt->give_node_coord2d (gx,gy,eid);
  giveloccoord (gx,gy,x);
  //  direction vector
  dirvect (s,gx,gy);
  Mt->give_elemnodes (eid,nodes);
  Mt->give_code_numbers (eid,cn.a);
  eldispl (lcid,eid,r.a,cn.a,ndofe);

  //  transformation of displacement vector
  long transf = Mt->locsystems (nodes);
  if (transf>0){
    allocv (ndofe,aux);
    allocm (ndofe,ndofe,tmat);
    transf_matrix (nodes,tmat);
    //locglobtransf (aux,r,tmat);
    lgvectortransf (aux,r,tmat);
    copyv (aux,r);
    destrv (aux);
    destrm (tmat);
  }
  
  mainip_strains (lcid,eid,0,0,x,s,r);
}

/**
   function computes strains in integration points of element

   @param lcid - load case id
   @param eid - element id
   @param ri - row index
   @param ci - column index

   10.5.2002
*/
void barelq2d::mainip_strains (long lcid,long eid,long ri,long ci,vector &x,vector &s,vector &r)
{
  long i,ii,ipp;
  double xi,jac;
  vector gp,w,eps;
  matrix gm;

  for (ii=0;ii<nb;ii++){
    if (intordsm[ii][ii]==0)  continue;

    allocv (intordsm[ii][ii],gp);
    allocv (intordsm[ii][ii],w);
    allocv (ncomp[ii],eps);
    allocm (ncomp[ii],ndofe,gm);

    gauss_points (gp.a,w.a,intordsm[ii][ii]);

    ipp=Mt->elements[eid].ipp[ri+ii][ci+ii];
    for (i=0;i<intordsm[ii][ii];i++){
      xi=gp[i];

      geom_matrix (gm,x,s,xi,jac);
      mxv (gm,r,eps);

      Mm->storestrain (lcid,ipp,cncomp[ii],ncomp[ii],eps);
      ipp++;
    }

    destrm (gm);  destrv (eps);  destrv (w);  destrv (gp);
  }

}

/**
   function computes strains in nodes of element

   @param lcid - load case id
   @param eid - element id
   @param ri,ci - row and column indices
   
   10.5.2002
*/
void barelq2d::nod_strains_ip (long lcid,long eid,long ri,long ci)
{
  long i,j;
  ivector ipnum(nne),nod(nne);
  vector eps(tncomp);
  
  //  numbers of integration points closest to nodes
  nodipnum (eid,ri,ci,ipnum);
  
  //  node numbers of the element
  Mt->give_elemnodes (eid,nod);
  
  for (i=0;i<nne;i++){
    //  strains at the closest integration point
    Mm->givestrain (lcid,ipnum[i],eps);
    
    //  storage of strains to the node
    j=nod[i];
    Mt->nodes[j].storestrain (lcid,0,eps);
  }
  
}


/**
   function computes nodal strains directly
   
   @param lcid - load case id
   @param eid - element id
   @param stra - array for strain components
   
   JK, 25.9.2004
*/
void barelq2d::nod_strains_comp (long lcid,long eid,double **stra)
{
  long i,j;
  double jac;
  ivector cn(ndofe),nodes(nne);
  vector gx(nne),gy(nne),x(nne),s(2),nxi(nne),r(ndofe),eps(tncomp),aux;
  matrix tmat,gm(tncomp,ndofe);
  
  //  node coordinates
  Mt->give_node_coord2d (gx,gy,eid);
  giveloccoord (gx,gy,x);
  //  computation of direction vector
  dirvect (s,gx,gy);
  //  node numbers
  Mt->give_elemnodes (eid,nodes);
  //  code numbers of the element
  Mt->give_code_numbers (eid,cn.a);
  //  nodal displacements
  eldispl (lcid,eid,r.a,cn.a,ndofe);
  
  //  transformation of displacement vector
  long transf = Mt->locsystems (nodes);
  if (transf>0){
    allocv (ndofe,aux);
    allocm (ndofe,ndofe,tmat);
    transf_matrix (nodes,tmat);
    //locglobtransf (aux,r,tmat);
    lgvectortransf (aux,r,tmat);
    copyv (aux,r);
    destrv (aux);
    destrm (tmat);
  }
  
  //  natural coordinates of element nodes
  nodecoord (nxi);
  
  //  loop over nodes
  for (i=0;i<nne;i++){
    //  geometric matrix
    geom_matrix (gm,x,s,nxi[i],jac);
    //  strain computation
    mxv (gm,r,eps);
    
    for (j=0;j<eps.n;j++){
      stra[i][j]=eps[j];
    }
  }

}

/**
   function computes all strain components at all integration points
   
   @param lcid - load case id
   @param eid - element id
   
   JK, 26.9.2004
*/
void barelq2d::res_allip_strains (long lcid,long eid)
{
  //  all strain components at all integration points
  allip_strains (lcid,eid,0,0);
}

/**
   function assembles all values at all integration points
   
   @param lcid - load case id
   @param eid - element id
   @param ri,ci - row and column indices
   
   JK, 25.9.2004
*/
void barelq2d::allip_strains (long lcid,long eid,long ri,long ci)
{
  //  blocks of strain components at integration points
  res_mainip_strains (lcid,eid);
}


/**
   function computes strains at strain points
   
   @param lcid - load case id
   @param eid - element id
   @param ri,ci - row and column indices
   
   JK
*/
void barelq2d::strains (long lcid,long eid,long ri,long ci)
{
  long i,naep,ncp,sid;
  double **stra;
  vector coord,eps;

  switch (Mm->stra.tape[eid]){
  case nowhere:{
    break;
  }
  case intpts:{
    allip_strains (lcid,eid,ri,ci);
    break;
  }
  case enodes:{
    nod_strains_ip (lcid,eid,ri,ci);
    break;
  }
  case userdefined:{
    //  number of auxiliary element points
    naep = Mm->stra.give_naep (eid);
    ncp = Mm->stra.give_ncomp (eid);
    sid = Mm->stra.give_sid (eid);
    allocv (ncp,eps);
    allocv (1,coord);
    for (i=0;i<naep;i++){
      Mm->stra.give_aepcoord (sid,i,coord);

      if (Mp->strainaver==0)
	//appval (coord[0],0,ncp,eps,stra);
      if (Mp->strainaver==1)
	//appstrain (lcid,eid,coord[0],0,ncp,eps);

      Mm->stra.storevalues(lcid,eid,i,eps);
    }
    destrv (eps);
    destrv (coord);
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown strain point is required in function barelq2d::strains (file %s, line %d).\n",__FILE__,__LINE__);
  }
  }

  if (Mp->strainaver==0){
    for (i=0;i<nne;i++){
      delete [] stra[i];
    }
    delete [] stra;
  }
}





/**
   function assembles natural coordinates of nodes of element

   @param xi - array containing natural coordinates xi

   10.5.2002
*/
void barelq2d::nodecoord (vector &xi)
{
  xi[0] = -1.0;
  xi[1] =  1.0;
  xi[2] =  0.0;
}


/**
   function returns numbers of integration point closest to element nodes
   
   @param eid - element id
   @param ri,ci - row and column indices
   @param ipnum - array of numbers
   
   JK, 25.9.2004
*/
void barelq2d::nodipnum (long eid,long ri,long ci,ivector &ipnum)
{
  long i;
  
  i=Mt->elements[eid].ipp[ri][ci];

  ipnum[0]=i;
  ipnum[1]=i+2;
  ipnum[2]=i+1;
}





/**
   function computes stresses at integration points
   
   @param lcid - load case id
   @param eid - element id
   
   JK
*/
void barelq2d::res_mainip_stresses (long lcid,long eid)
{
  long i;
  
  //  loop over blocks
  for (i=0;i<nb;i++){
    mainip_stresses (lcid,eid,0,0,i);
  }
}

/**
   function computes stresses in integration points of element

   @param eid - element id
   @param ri - row index
   @param ci - column index
   @param ii - number of actual block

   10.5.2002
*/
void barelq2d::mainip_stresses (long lcid,long eid,long ri,long ci,long ii)
{
  long i,jj,ipp;
  double xi;
  vector gp,w,eps,epst,epstt,sig,auxsig;
  matrix d(tncomp,tncomp);
  
  allocv (ncomp[ii],sig);
  allocv (ncomp[ii],auxsig);
  allocv (intordsm[ii][ii],gp);
  allocv (intordsm[ii][ii],w);
  
  gauss_points (gp.a,w.a,intordsm[ii][ii]);
  ipp=Mt->elements[eid].ipp[ri+ii][ci+ii];
  
  for (i=0;i<intordsm[ii][ii];i++){
    xi=gp[i];
    
    Mm->matstiff (d,ipp);
    
    fillv (0.0,sig);
    for (jj=0;jj<nb;jj++){
      allocv (ncomp[jj],eps);
      
	//  block of strains
      Mm->givestrain (lcid,ipp,cncomp[jj],ncomp[jj],eps);
      
      //  stress contributions
      mxv (d,eps,auxsig);
      //  summation of contributions
      addv (auxsig,sig,sig);

      destrv (eps);
    }
    
    //  storage of block of stress
    Mm->storestress (lcid,ipp,cncomp[ii],ncomp[ii],sig);
    
    ipp++;
  }
  
  destrv (w);  destrv (gp);  destrv (auxsig);  destrv (sig);
  
}

/**
   function computes stresses at nodes of element

   @param lcid - load case id
   @param eid - element id
   @param ri,ci - row and column indices
   
   10.5.2002
*/
void barelq2d::nod_stresses_ip (long lcid,long eid,long ri,long ci)
{
  long i,j;
  ivector ipnum(nne),nod(nne);
  vector sig(tncomp);
  
  //  numbers of integration points closest to nodes
  nodipnum (eid,ri,ci,ipnum);
  
  //  node numbers of the element
  Mt->give_elemnodes (eid,nod);
  
  for (i=0;i<nne;i++){
    //  stresses at the closest integration point
    Mm->givestress (lcid,ipnum[i],sig);
    
    //  storage of stresses to the node
    j=nod[i];
    Mt->nodes[j].storestress (lcid,0,sig);
  }
  
}

/**
   function computes nodal stresses directly
   
   @param lcid - load case id
   @param eid - element id
   @param stra - array for strain components
   
   JK, 25.9.2004
*/
void barelq2d::nod_stresses_comp (long lcid,long eid,long ri,long ci,double **stra,double **stre)
{
  long i,j,ipp;
  vector eps(tncomp),sig(tncomp);
  matrix d(tncomp,tncomp);
  
  //  number of the first integration point on the element
  ipp=Mt->elements[eid].ipp[ri][ci];
  //  stiffness matrix of the material
  Mm->matstiff (d,ipp);
  
  //  loop over nodes
  for (i=0;i<nne;i++){
    for (j=0;j<eps.n;j++){
      eps[j]=stra[i][j];
    }
    mxv (d,eps,sig);
    for (j=0;j<eps.n;j++){
      stre[i][j]=sig[j];
    }
  }
}

/**
   function computes all stress components at all integration points
   
   @param lcid - load case id
   @param eid - element id
   
   JK
*/
void barelq2d::res_allip_stresses (long lcid,long eid)
{
  //  all stress components at all intergation points
  allip_stresses (lcid,eid,0,0);
}

/**
   function computes all stress components at all integration points
   
   @param lcid - load case id
   @param eid - element id
   @param ri,ci - row and column indices
   
   JK
*/
void barelq2d::allip_stresses (long lcid,long eid,long ri,long ci)
{
  //  blocks of stress components at integration points
  res_mainip_stresses (lcid,eid);
}



void barelq2d::stresses (long lcid,long eid,long ri,long ci)
{
  long i,naep,ncp,sid;
  vector coord,sig;

  switch (Mm->stre.tape[eid]){
  case nowhere:{
    break;
  }
  case intpts:{
    allip_stresses (lcid,eid,ri,ci);
    break;
  }
  case enodes:{
    nod_stresses_ip (lcid,eid,ri,ci);
    break;
  }
  case userdefined:{
    //  number of auxiliary element points
    naep = Mm->stre.give_naep (eid);
    ncp = Mm->stre.give_ncomp (eid);
    sid = Mm->stre.give_sid (eid);
    allocv (ncp,sig);
    allocv (1,coord);
    for (i=0;i<naep;i++){
      Mm->stre.give_aepcoord (sid,i,coord);

      if (Mp->stressaver==0)
	//appval (coord[0],0,ncp,sig,stre);
      if (Mp->stressaver==1)
	//appstress (lcid,eid,coord[0],0,ncp,sig);

      Mm->stre.storevalues(lcid,eid,i,sig);
    }
    destrv (sig);
    destrv (coord);
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown stress point is required in function barelq2d::stresses (file %s, line %d).\n",__FILE__,__LINE__);
  }
  }

}






/**
   function computes other values in nodes of element

   @param lcid - load case id
   @param eid - element id
   
   10.5.2002
*/
void barelq2d::nod_eqother_ip (long lcid,long eid,long ri,long ci)
{
  long i,j,ncompo;
  ivector ipnum(nne),nod(nne);
  vector eqother;
  
  //  numbers of integration points closest to nodes
  nodipnum (eid,ri,ci,ipnum);
  
  //  node numbers of the element
  Mt->give_elemnodes (eid,nod);
  
  for (i=0;i<nne;i++){
    //  strains at the closest integration point
    //Mm->givestrain (lcid,ipnum[i],eps);
    
    ncompo = Mm->givencompeqother (ipnum[i],0);
    allocv (ncompo,eqother);
    Mm->giveeqother (ipnum[i],0,ncompo,eqother.a);
    
    //  storage of other values to the node
    j=nod[i];
    Mt->nodes[j].storeother (lcid,0,ncompo,eqother);
    
    destrv (eqother);
  }
}






/**

   load vector is obtained after premultiplying load %matrix
   by nodal load values
   
   @param eid - number of element
   @param lm - load %matrix
   
   JK, 12.4.2003
*/
void barelq2d::load_matrix (long eid,matrix &lm)
{
  /*
  long i;
  double jac,xi,area;
  ivector nodes(nne);
  vector x(nne),y(nne),w(intordmm),gp(intordmm);
  matrix n(napfun,ndofe);
  
  Mt->give_elemnodes (eid,nodes);
  Mc->give_area (eid,area);

  Mt->give_node_coord2d (x,y,eid);
  

  gauss_points (gp.a,w.a,intordmm);
  
  fillm (0.0,lm);

  for (i=0;i<intordmm;i++){
    xi=gp[i];
    
    jac_1d (jac,x,xi);
    bf_matrix (n,xi);
    
    jac*=w[i]*area;
    
    nnj (lm.a,n.a,jac,n.m,n.n);
  }
  */
}

/**
   function computes internal forces

   @param lcid - load case id
   @param eid - element id
   @param ri,ci - row and column indices
   @param ifor - vector of internal forces
   @param x - x-coordinates in local system
   @param s - direction %vector

   12.8.2001
*/
void barelq2d::internal_forces (long lcid,long eid,long ri,long ci,vector &ifor,vector &x,vector &s)
{
  integratedquant iq;
  iq=locstress;

  //  computation of stresses
  compute_nlstress (lcid,eid,ri,ci);

  //  integration of stresses over the element
  elem_integration (iq,lcid,eid,ri,ci,ifor,x,s);
}



/**
   function computes internal forces for nonlocal models

   @param lcid - load case id
   @param eid - element id
   @param ri,ci - row and column indices
   @param ifor - vector of internal forces
   @param x - x-coordinates in local system
   @param s - direction %vector

   12.8.2001
*/
void barelq2d::nonloc_internal_forces (long lcid,long eid,long ri,long ci,vector &ifor,vector &x,vector &s)
{
  integratedquant iq;
  iq=nonlocstress;

  //  computation of stresses
  compute_nonloc_nlstress (lcid,eid,ri,ci);

  //  integration of stresses over the element
  elem_integration (iq,lcid,eid,ri,ci,ifor,x,s);
}



/**
   function computes increment of internal forces
   
   @param lcid - load case id
   @param eid - element id
   @param ri,ci - row and column indices
   @param ifor - vector of internal forces
   @param x - x-coordinates in local system
   @param s - direction %vector
   
   JK, 29.4.2008
*/
void barelq2d::incr_internal_forces (long lcid,long eid,long ri,long ci,vector &ifor,vector &x,vector &s)
{
  integratedquant iq;
  iq=stressincr;

  //  computation of stresses
  compute_nlstressincr (lcid,eid,ri,ci);
  
  //  integration of stresses over the element
  elem_integration (iq,lcid,eid,ri,ci,ifor,x,s);
}



/**
   function computes nodal forces caused by temperature changes
   
   @param eid - element id
   @param ri,ci - row and column indices
   @param nfor - array containing nodal forces
   @param x - x-coordinates in local system
   @param s - direction %vector
   
   30.11.2002, JK
*/
void barelq2d::eigstrain_forces (long lcid,long eid,long ri,long ci,vector &nfor,vector &x,vector &s)
{
  integratedquant iq;
  iq=eigstress;

  //  computation of eigenstresses
  compute_eigstress (lcid,eid,ri,ci);
  
  //  integration of stresses over the element
  elem_integration (iq,lcid,eid,ri,ci,nfor,x,s);
}



void barelq2d::res_stiffness_matrix (long eid,matrix &sm)
{
  long transf;
  ivector nodes(nne);
  vector gx(nne),gy(nne),x(nne),s(2);
  matrix tmat (ndofe,ndofe);
  
  Mt->give_node_coord2d (gx,gy,eid);
  Mt->give_elemnodes (eid,nodes);
  
  giveloccoord (gx,gy,x);
  //  computation of direction vector
  dirvect (s,gx,gy);

  stiffness_matrix (eid,0,0,sm,x,s);
  

  //TKr 31.10.2006 ??!!!
  //give_loc_glob_tmat(gx,gy,tmat);
  //lgmatrixtransf (sm,tmat);

  //  transformation of stiffness matrix
  transf = Mt->locsystems (nodes);
  if (transf>0){
    transf_matrix (nodes,tmat);
    glmatrixtransf (sm,tmat);
  }
}

void barelq2d::res_mass_matrix (long eid,matrix &sm)
{
  vector gx(nne),gy(nne),x(nne);
  Mt->give_node_coord2d (gx,gy,eid);
  
  giveloccoord (gx,gy,x);
  // mass_matrix ();
}



/**
   function computes resulting internal forces

   @param lcid - load case id
   @param eid - element id
   @param ifor - vector of internal forces

   12.8.2001
*/
void barelq2d::res_internal_forces (long lcid,long eid,vector &ifor)
{
  vector gx(nne),gy(nne),x(nne),s(2);
  Mt->give_node_coord2d (gx,gy,eid);
  
  giveloccoord (gx,gy,x);
  //  computation of direction vector
  dirvect (s,gx,gy);

  internal_forces (lcid,eid,0,0,ifor,x,s);
}



/**
   function computes resulting increment of internal forces

   @param lcid - load case id
   @param eid - element id
   @param ifor - vector of internal forces

   TKo 7.2008
*/
void barelq2d::res_nonloc_internal_forces (long lcid,long eid,vector &ifor)
{
  vector gx(nne),gy(nne),x(nne),s(2);
  Mt->give_node_coord2d (gx,gy,eid);
  
  giveloccoord (gx,gy,x);
  //  computation of direction vector
  dirvect (s,gx,gy);

  incr_internal_forces (lcid,eid,0,0,ifor,x,s);
}



/**
   function computes resulting increment of internal forces

   @param lcid - load case id
   @param eid - element id
   @param ifor - vector of internal forces

   TKo 7.2008
*/
void barelq2d::res_incr_internal_forces (long lcid,long eid,vector &ifor)
{
  vector gx(nne),gy(nne),x(nne),s(2);
  Mt->give_node_coord2d (gx,gy,eid);
  
  giveloccoord (gx,gy,x);
  //  computation of direction vector
  dirvect (s,gx,gy);

  incr_internal_forces (lcid,eid,0,0,ifor,x,s);
}



/**
   function computes resulting internal forces

   @param lcid - load case id
   @param eid - element id
   @param nfor - vector of nodal forces

   12.8.2001
*/
void barelq2d::res_eigstrain_forces (long lcid,long eid,vector &nfor)
{
  vector gx(nne),gy(nne),x(nne),s(2);
  Mt->give_node_coord2d (gx,gy,eid);
  
  giveloccoord (gx,gy,x);
  //  computation of direction vector
  dirvect (s,gx,gy);

  eigstrain_forces (lcid,eid,0,0,nfor,x,s);
}



/**
   function integrates selected quantity over the finite element
   it results in nodal values
   
   @param iq - type of integrated quantity (see alias.h)
   @param lcid - number of load case
   @param eid - element id
   @param ri,ci - row and column indices
   @param nv - nodal values
   @param x - local x coordinate
   @param s - directional vector
   
   TKo, 7.2008
*/
void barelq2d::elem_integration (integratedquant iq, long lcid,long eid,long ri,long ci,vector &nv,vector &x,vector &s)
{
  long i,ii,ipp;
  double xi,jac,area;
  vector w,gp,ipv,contr(ndofe);
  matrix gm;
  
  Mc->give_area (eid,area);
  fillv (0.0,nv);
  
  for (ii=0;ii<nb;ii++){
    allocv (intordsm[ii][ii],gp);
    allocv (intordsm[ii][ii],w);
    allocm (ncomp[ii],ndofe,gm);
    allocv (ncomp[ii],ipv);
    
    gauss_points (gp.a,w.a,intordsm[ii][ii]);
    ipp=Mt->elements[eid].ipp[ri+ii][ci+ii];
    
    for (i=0;i<intordsm[ii][ii];i++){
      xi=gp[i];
      
      //  function assembles required quantity at integration point
      Mm->givequantity (iq,lcid,ipp,cncomp[ii],ipv);
      
      //  strain-displacement (geometric) matrix
      geom_matrix (gm,x,s,xi,jac);

      //  contribution to the nodal values
      mtxv (gm,ipv,contr);
      
      cmulv (jac*w[i]*area,contr);
      
      //  summation
      addv(contr, nv, nv);
      
      ipp++;
    }
  }
  destrv (ipv);  destrm (gm);  destrv (w);  destrv (gp);
}



/**
   function computes correct stresses at integration points on element

   @param lcid - number of load case
   @param eid - element id
   @param ri,ci - row and column indices
   
   TKo 7.2008
*/
void barelq2d::compute_nlstress (long lcid,long eid,long ri,long ci)
{
  long i,j,ii,jj,ipp;
  
  for (ii=0;ii<nb;ii++){
    for (jj=0;jj<nb;jj++){
      if (intordsm[ii][jj]==0)  continue;
      
      ipp=Mt->elements[eid].ipp[ri+ii][ci+jj];
      
      for (i=0;i<intordsm[ii][jj];i++){
	for (j=0;j<intordsm[ii][jj];j++){
	  
	  //  computation of correct stresses
	  if (Mp->strcomp==1)
	    Mm->computenlstresses (ipp);
	  
	  ipp++;
	}
      }
    }
  }
}



/**
   function computes correct stresses at integration points on element

   @param lcid - number of load case
   @param eid - element id
   @param ri,ci - row and column indices
   
   TKo 7.2008
*/
void barelq2d::compute_nlstressincr (long lcid,long eid,long ri,long ci)
{
  long i,j,ii,jj,ipp;
  
  for (ii=0;ii<nb;ii++){
    for (jj=0;jj<nb;jj++){
      if (intordsm[ii][jj]==0)  continue;
      
      ipp=Mt->elements[eid].ipp[ri+ii][ci+jj];
      
      for (i=0;i<intordsm[ii][jj];i++){
	for (j=0;j<intordsm[ii][jj];j++){
	  
	  //  computation of correct stresses
	  if (Mp->strcomp==1)
            Mm->computenlstressesincr (ipp);

	  ipp++;
	}
      }
    }
  }
}



/**
   function computes local values which will be used for averageing 
   in nonlocal models at integration points. Mp->nonlocphase have to be 1.

   @param lcid - number of load case
   @param eid - element id
   @param ri,ci - row and column indices
   
   TKo, 7.2008
*/
void barelq2d::local_values (long lcid,long eid,long ri,long ci)
{
  long i,j,ii,jj,ipp;
  
  for (ii=0;ii<nb;ii++){
    for (jj=0;jj<nb;jj++){
      if (intordsm[ii][jj]==0)  continue;
      
      ipp=Mt->elements[eid].ipp[ri+ii][ci+jj];
      
      for (i=0;i<intordsm[ii][jj];i++){
	for (j=0;j<intordsm[ii][jj];j++){
	  
	  //  computation of correct stresses
	  if (Mp->strcomp==1)
	    Mm->computenlstresses (ipp);
	  
	  ipp++;
	}
      }
    }
  }
}



/**
   function computes nonlocal correct stresses at integration points on element
   
   @param lcid - number of load case
   @param eid - element id
   @param ri,ci - row and column indices
   
   TKo 7.2008
*/
void barelq2d::compute_nonloc_nlstress (long lcid,long eid,long ri,long ci)
{
  long i,j,ii,jj,ipp;
  
  for (ii=0;ii<nb;ii++){
    for (jj=0;jj<nb;jj++){
      if (intordsm[ii][jj]==0)  continue;
      
      ipp=Mt->elements[eid].ipp[ri+ii][ci+jj];
      
      for (i=0;i<intordsm[ii][jj];i++){
	for (j=0;j<intordsm[ii][jj];j++){
	  
	  //  computation of correct stresses
	  if (Mp->strcomp==1)
	    Mm->compnonloc_nlstresses (ipp);
	  
	  ipp++;
	}
      }
    }
  }
}



/**
   function computes nonlocal correct stresses at integration points on element
   
   @param lcid - number of load case
   @param eid - element id
   @param ri,ci - row and column indices
   
   TKo, 7.2008
*/
void barelq2d::compute_eigstress (long lcid,long eid,long ri,long ci)
{
  long i,j,ii,jj,ipp;
  vector eigstr(tncomp),sig(tncomp);
  matrix d(tncomp,tncomp);
  
  for (ii=0;ii<nb;ii++){
    for (jj=0;jj<nb;jj++){
      if (intordsm[ii][jj]==0)  continue;
      
      ipp=Mt->elements[eid].ipp[ri+ii][ci+jj];
      
      for (i=0;i<intordsm[ii][jj];i++){
	for (j=0;j<intordsm[ii][jj];j++){
	  
	  Mm->giveeigstrain (ipp,eigstr);
	  
	  //  matrix of stiffness of the material
	  Mm->matstiff (d,ipp);
	  
	  mxv (d,eigstr,sig);
	  
	  Mm->storeeigstress (ipp,sig);
	  
	  ipp++;
	}
      }
    }
  }
}



void barelq2d::inicipval(long eid, long ri, long ci, matrix &nodval, inictype *ictn)
{
  long i, j, k, ipp;
  long ii, jj, nv = nodval.n;
  long nstra;
  double xi, ipval;
  vector w, gp, anv(3);

  nstra = 0;
  for (j = 0; j < nv; j++) // for all initial values
  {
    for(i = 0; i < nne; i++) // for all nodes on element
      anv[i] = nodval[i][j];
    for (ii = 0; ii < nb; ii++)
    {
      for (jj = 0; jj < nb; jj++)
      {
        ipp=Mt->elements[eid].ipp[ri+ii][ci+jj];
        if (intordsm[ii][jj] == 0)
          continue;
        allocv (intordsm[ii][jj],gp);
        allocv (intordsm[ii][jj],w);
        gauss_points (gp.a,w.a,intordsm[ii][jj]);
        for (k = 0; k < intordsm[ii][jj]; k++)
        {
          xi=gp[k];
          //  value in integration point
          ipval = approx (xi,anv);
          if ((ictn[i] & inistrain) && (j < Mm->ip[ipp].ncompstr))
          {
            Mm->ip[ipp].strain[j] += ipval;
            ipp++;
            continue;
          }
          if ((ictn[i] & inistress) && (j < nstra + Mm->ip[ipp].ncompstr))
          {
            Mm->ip[ipp].stress[j] += ipval;
            ipp++;
            continue;
          }
          if ((ictn[i] & iniother) && (j < nv))
          {
            Mm->ip[ipp].other[j] += ipval;
            ipp++;
            continue;
          }
          ipp++;
        }
        destrv (gp);  destrv (w);
      }
    }
    if (ictn[i] & inistrain) nstra++;
  }
}


void barelq2d::intpointval (long eid,vector &nodval,vector &ipval)
{
  long ii,jj,i,k,ipp;
  double xi;
  vector w,gp;

  k=0;
  for (ii = 0; ii < nb; ii++)
  {
    for (jj = 0; jj < nb; jj++)
    {
      ipp=Mt->elements[eid].ipp[ii][jj];
      if (intordsm[ii][jj] == 0)
        continue;
      allocv (intordsm[ii][jj],gp);
      allocv (intordsm[ii][jj],w);
      gauss_points (gp.a,w.a,intordsm[ii][jj]);
      for (i = 0; i < intordsm[ii][jj]; i++)
      {
        xi=gp[i];
        //  value in integration point
        ipval[k] = approx (xi,nodval);
        k++;
        ipp++;
      }
      destrv (gp);  destrv (w);
    }
  }
}

void barelq2d::intpointval2 (long eid,vector &nodval,vector &ipval)
{
  long ii,jj,i,k;
  double xi;
  vector w,gp;
  vector modnodval(Bar2d->nne);
  
  for (i=0;i<Bar2d->nne;i++){
    modnodval[i]=nodval[i];
  }

  k=0;
  for (ii = 0; ii < nb; ii++){
    for (jj = 0; jj < nb; jj++){
      if (intordsm[ii][jj] == 0)
        continue;

      allocv (intordsm[ii][jj],gp);
      allocv (intordsm[ii][jj],w);

      gauss_points (gp.a,w.a,intordsm[ii][jj]);

      for (i = 0; i < intordsm[ii][jj]; i++){
        xi=gp[i];

        //  value in integration point
        ipval[k] = Bar2d->approx (xi,modnodval);

        k++;
      }
      destrv (gp);  destrv (w);
    }
  }
}



/**
   function assembles coordinates of integration points
   
   @param eid - element id
   @param ipp - integration point pointer
   @param ri - row index
   @param ci - column index
   @param ipcoord - array containing coordinates of integration points
   
   TKo, 7.2008
*/
void barelq2d::ipcoord (long eid,long ipp,long ri,long ci,vector &coord)
{
  long ii,jj,i,aipp;
  double xi;
  vector w,gp;
  vector x(nne),y(nne);

  Mt->give_node_coord2d (x,y,eid);
  for (ii = 0; ii < nb; ii++){
    for (jj = 0; jj < nb; jj++){
      if (intordsm[ii][jj] == 0)
        continue;
      //  integration point id
      aipp=Mt->elements[eid].ipp[ri+ii][ci+jj];
      allocv (intordsm[ii][jj],gp);
      allocv (intordsm[ii][jj],w);
      gauss_points (gp.a,w.a,intordsm[ii][jj]);
      
      for (i=0;i<intordsm[ii][jj];i++){
        xi=gp[i];
        if (aipp==ipp){
          coord[0]=approx (xi,x);
          coord[1]=approx (xi,y);
          coord[2]=0.0;
        }
        aipp++;
      }
      destrv (gp);  destrv (w);
    }
  }
}
