#include <string.h>
#include "cpsolver.h"
#include "mtsolver.h"
#include "global.h"
#include "globmat.h"
#include "mechprint.h"
#include "matrix.h"
#include "vector.h"
#include "gmatrix.h"
#include "gtopology.h"
#include "dloadcase.h"
#include "node.h"
#include "element.h"


/**
   Function solves non-linear MEFEL equations by Newton-Raphson method for time-dependent problems
   for growing structure
   
   @param lcid - load case id
   
   15.6.2007, TKr
*/
void solve_prob_constr_phases ()
{
  long lcid,ii,j,k,nm,mncd,mnce,ini;
  double dt,prev_timem,end_time;
  double *dr,*fb,*r,*f,*fp,*fi;
  double norf, norfa, err;  
  
  //  load case id must be equal to zero
  lcid=0;

  //  maximum number of iterations in inner loop
  ini = Mp->niilnr;
  //  required norm of vector of unbalanced forces
  err = Mp->errnr;
  //  number of mechanical degrees of freedom
  nm=Ndofm;
  
  /* **************************** */
  /*  vectors of mechanical part  */
  /* **************************** */
  fp = new double [nm];
  fb = new double [nm];
  fi = new double [nm];
  dr = new double [nm];
  
  //  initial values
  nullv (fp,nm);
  nullv (fb,nm);
  nullv (fi,nm);
  nullv (dr,nm);
  
  r = Lsrs->give_lhs (lcid);
  f = Lsrs->give_rhs (lcid);
  
  //  provizorni umisteni, bude v mefelinit
  Gtm->comp_domain_sizes ();
  Mb->alloc_sumcomp ();
  //  konec provizoria??!!

  //  initial time
  Mp->time = Mp->timecon.starttime ();
  prev_timem = Mp->time;
  Mp->timecon.take_values (Mp->timecon);
  //  end time
  end_time = Mp->timecon.endtime ();

  //  initiation of mechanical material models
  Mm->initmaterialmodels();

  Gtm->nm = new long [Gtm->nn];
  
  // **************************
  //  main iteration loop  ***
  // **************************
  ii=0;
  print_init(-1, "wt");
  print_step(lcid,ii,Mp->time,f);
  print_flush();

  //saving values for the first iteration
  Mt->lhs_save (r,1);

  do{
    
    //if (Mespr==1)  
    fprintf (stdout,"\n\n ------------------------------------------------------------------------------");
    //if (Mespr==1)  
    fprintf (stdout,"\n\n provadi se MEFEL,  mefel time %e",Mp->time);
    //if (Mespr==1)  
    //fprintf (stdout,"\n time increment %lf",dt);
    //if (Mespr==1)  
    fprintf (stdout,"\n -----------------------------------------------------------------------------\n");
    
            
      //  searching for new elements
      //  only new elements are switched on
      //  former elements are temporarily switched off
      mnce = Gtm->search_newelem (Mp->time,prev_timem);
    
      if (mnce!=0){
	//  attained displacements have to be saved on new elements
	//  otherwise stresses will not be correct
	Mt->initial_displ ();
      }
      
      //  searching for new DOFs
      //  only new DOFs are switched on
      //  former DOFs are temporarily switched off
      mncd = Gtm->search_newdofs (Mp->time,prev_timem);
      
      //  marking of elements switched on
      Gtm->update_elem (Mp->time);
      
      //  marking of nodes switched on
      Gtm->update_nodes ();
      
      //  marking of DOFs switched on
      Gtm->update_dofs (Mp->time);
      
      //  definition of DOFs
      //Ndofm = Gtm->codenum_generation (Out);
      nm=Ndofm;
      
      //  update of nodes and DOFs
      //if (mncd>0){
      if (Smat != NULL){
	delete Smat;
	Smat=NULL;
      }
      //}
      
      //  assembling of lhs array for new code numbers
      //if (mnce!=0 || mncd!=0){
      Mt->lhs_restore (r,nm);
      Mt->rhs_restore (fp,nm);
      //}
            
      Mm->updateipval();

      //fprintf (stdout,"\nright hand side\n\n");
      //  assembling of the right hand side (prescribed loading)
      mefel_right_hand_side (lcid,f);
      
      //  computation of sums of force components in particular directions
      //  it is used in special creep and consolidation models
      Mb->comp_sum (f);
      
      Mp->phase=1;
      //  computation of unbalanced forces which will be on the right hand side
      internal_forces (lcid,fi);
      
      for (j=0;j<nm;j++){
	fb[j]=f[j]-fp[j]+fi[j];
	fp[j]=f[j];
      }    
      
      //  assembling of tangent stiffness matrix
      //fprintf (stdout,"\nphase1 = stiffness_matrix\n\n");
      stiffness_matrix (lcid);
      
      //  solution of K(r).v=F
      Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);

      //  new displacement vector
      for (j=0;j<nm;j++){
	r[j]+=dr[j];
      }
      
      Mp->phase=2;

      //  update of computed values
      internal_forces (lcid,fi);

      norfa=ss(f,f,nm);
      for (j=0;j<nm;j++){
	fb[j] = f[j] - fi[j];
      }
      //  norm of vector of unbalanced forces
      norf=ss(fb,fb,nm);
      if (norfa > 0.0)
	norf /= norfa;
      
      if (norf<err){

	//  backup of nodal values because code numbers may be changed
	Mt->lhs_save (r,1);
	Mt->rhs_save (f);

	// computation of values required for output    
	compute_req_val (lcid);
	//  time increment
	prev_timem=Mp->time;
	Mp->time = Mp->timecon.newtime ();
	Mp->timecon.take_values (Mp->timecon);

	if (Mp->timecon.isitimptime ()==1)
          mtsolver_save (r,fp,ii,Mp->time,dt,nm);
	
	print_step(lcid, ii, Mp->time, f);
	print_flush();
	ii++;	
      }
      
      // iteration of unbalanced forces caused by time independent models
      //  internal iteration loop
      else{
	for (j=0;j<ini;j++)
	  {
	    
	    if(Mp->nrsolv == fullnewton){
	      //cleaning of stiffness matrix
	      if (Smat != NULL){
		delete Smat;
		Smat=NULL;
	      }
	      //assembling of tangent stiffness matrix for fullnewton
	      stiffness_matrix (lcid);
	    }
	    
	    //  solution of K(r).v=F
	    Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);

	    for (k=0;k<nm;k++)
	      r[k]+=dr[k];
	    
	    //  computation of internal forces
	    internal_forces (lcid,fi);
	    
	    //  vector of unbalanced forces
	    for (k=0;k<nm;k++)
	      fb[k]=f[k]-fi[k];
	    
	    //  norm of vector of unbalanced forces
	    norf=ss(fb,fb,nm);
	    if (norfa > 0.0)
	      norf /= norfa;
	    
	    //if (Mespr==1) 
	    fprintf (stdout,"\n Time increment %lf   inner loop j %ld     norf=%e",Mp->time,j,norf);
	    if (norf<err)  break;
	  }
	if (j==ini || norf>err)
	  {
	    //if (Mespr==1) 
	    fprintf (stdout,"\n Iteration process does not converge req. err = %e, reached err = %e", err,norf);
	    
	    //backup of norf in case of unbalanced internal forces
	    err = norf;

	    //  backup of nodal values because code numbers may be changed
	    Mt->lhs_save (r,1);
	    Mt->rhs_save (f);
	    
	    // equilibrium state was reached not
	    compute_req_val (lcid);
	    //  time increment
	    prev_timem = Mp->time;
	    Mp->time = Mp->timecon.newtime ();
	    Mp->timecon.take_values (Mp->timecon);
	    
	    if (Mp->timecon.isitimptime ()==1)
              mtsolver_save (r,fp,ii,Mp->time,dt,nm);

	    print_step(lcid, ii, Mp->time, f);
	    print_flush();
	    ii++;
	  }
	else
	  {
	    //  backup of nodal values because code numbers may be changed
	    Mt->lhs_save (r,1);
	    Mt->rhs_save (f);
	    
	    // equilibrium state was reached
	    compute_req_val (lcid);
	    //  time increment
	    prev_timem = Mp->time;
	    Mp->time = Mp->timecon.newtime ();
	    Mp->timecon.take_values (Mp->timecon);

	    if (Mp->timecon.isitimptime ()==1)
              mtsolver_save (r,fp,ii,Mp->time,dt,nm);


	    print_step(lcid, ii, Mp->time, f);
	    print_flush();
	    ii++;
	  }
      }
      
      if (Mespr==1){
	//cleaning once again for more free memmory
	if (Smat != NULL){
	  delete Smat;
	  Smat=NULL;
	}
      }
  }while(Mp->time<=end_time);
  
  print_close ();
  
  delete [] fi;
  delete [] fb;
  delete [] fp;
  delete [] dr;
}




/**
   function solves problem of construction phases
   
   only one load case may be defined, one load case may contain
   several effects which are activated by time functions
   variable Mb->nlc must be equal to 1
   
   6.2.2006, JK
*/
void solve_prob_constr_phases_old ()
{
  long i,j,k,nm,ncd,nce,lcid,idcp,ini,nsts;
  double prev_time,end_time,zero,*f,*fi,*fb,*fp,*r,*dr,*lhsb,*fl;
  double err,dt,dtmin,dtdef,norfa,norf;
  char fname[1020];
  char *path;
  char *name;
  char *suffix;
  FILE *aux;
  matrix lsm_a(3,3);
  vector lsm_r(3), lsm_l(3);
  
  //  load case id must be equal to zero
  lcid=0;
  
  //  maximum number of iterations in inner loop
  ini = Mp->niilnr;
  //  required norm of vector of unbalanced forces
  err = Mp->errnr;
  //  computer zero
  zero = Mp->zero;
  
  //  number of mechanical degrees of freedom
  nm=Ndofm;

  fp = new double [nm];
  nullv (fp,nm);
  fb = new double [nm];
  nullv (fb,nm);
  fi = new double [nm];
  nullv (fi,nm);
  fl = new double [nm];
  nullv (fl,nm);
  dr = new double [nm];
  nullv (dr,nm);
  lhsb = new double [nm];
  nullv (lhsb,nm);

  //  vector of solution
  r = Lsrs->give_lhs (lcid);
  //  vector of the right hand side
  f = Lsrs->give_rhs (lcid);
  
  //  initial time
  Mp->time=Mp->timecon.starttime ();
  prev_time=Mp->time;
  //  end time
  end_time = Mp->timecon.endtime ();
  //  initial time increment
  dt = Mp->timecon.initialtimeincr ();
  
  
  //  provizorni umisteni, bude v mefelinit
  Gtm->comp_domain_sizes ();
  Mb->alloc_sumcomp ();
  //  konec provizoria
  
  //  initiation of mechanical material models
  Mm->initmaterialmodels();

  nullv (fp,nm);
  i=0;
  //  number of successful time steps (number of steps without inner iterations)
  nsts=0;
  
  //BEGIN OF GRAPHICS:
  //print_init(-1, "wt");
  //print_step(lcid, i, Mp->time, f);
  //print_flush();
  //print_close();
  //END OF GRAPHICS

  //FILE *kon;
  //kon = fopen ("kontrolavondr","w");
  //kon = fopen ("caskon","w");
  
  
  Gtm->nm = new long [Gtm->nn];
  
  print_init(-1, "wt");
  print_step(lcid, i, Mp->time, f);
  print_flush();
  
  // ***************************
  //   main iteration loop  ****
  // ***************************
  do{
    //new time increment
    Mp->time = Mp->timecon.newtime (dt);
    //  new step number
    i++;
    //  indicator of strain computation
    //  it means, strains at integration points have not been computed
    Mp->strainstate=0;
    //  indicator of stress computation
    //  it means, stresses at integration points have not been computed
    Mp->stressstate=0;
    //  indicator of computation of other array
    //  it means, components of array other have not been computed
    Mp->otherstate=0;
    
    //  backup of attained nodal values
    for (j=0;j<nm;j++){
      lhsb[j]=r[j];
    }

    fprintf (stdout,"\n time %e",Mp->time);

    //  searching for new elements
    //  only new elements are switched on
    //  former elements are temporarily switched off
    nce = Gtm->search_newelem (Mp->time,prev_time);
    
    if (nce!=0){
      //  attained displacements have to be saved on new elements
      //  otherwise stresses will not be correct
      Mt->initial_displ ();
    }
    
    //  searching for new DOFs
    //  only new DOFs are switched on
    //  former DOFs are temporarily switched off
    ncd = Gtm->search_newdofs (Mp->time,prev_time);

    /***************************************************/
    //////BEGIN OF GRAPHICS
    /*
    long graphics=0;
    if ((nce!=0 || ncd!=0) && graphics==1){
      //if (nce!=0 || ncd!=0){
      // ************************
      //  correction of graphic
      // ************************
      
      
      //fprintf (kon,"\n cas %le  grafika",Mp->time);
      //for (j=0;j<Gtm->nn;j++){
      //fprintf (kon,"\n uzel %6ld",j+1);
      //long ndofn=Gtm->gnodes[j].ndofn;
      //for (long k=0;k<ndofn;k++){
      //fprintf (kon," %ld",Gtm->gnodes[j].cn[k]);
      //}
      //}

      //  generation of new code numbers
      Ndofm = Gtm->gencodnum ();
      n=Ndofm;
      
      //  cleaning of the stiffness matrix
      if (Smat != NULL){
	delete Smat;
	Smat=NULL;
      }
      

      //fprintf (kon,"\n cas %le  grafika",Mp->time);
      //for (j=0;j<Gtm->nn;j++){
      //fprintf (kon,"\n uzel %6ld",j+1);
      //long ndofn=Gtm->gnodes[j].ndofn;
      //for (long k=0;k<ndofn;k++){
      //fprintf (kon," %ld",Gtm->gnodes[j].cn[k]);
      //}
      //}
      
      //  allocation and assembling of the stiffness matrix
      stiffness_matrix (lcid);
      
      //  nodal forces caused by attained displacements
      rhs_graphic (fb,lcid=0,n);
      
      //  solution of system of equations
      Smat->solve_system (dr,fb);
      
      //  storage of auxiliary displacements to nodes
      Mt->lhs_save (dr,2);
    }
    */
    //////END OF GRAPHICS
    /***************************************************/


    //  marking of elements switched on
    Gtm->update_elem (Mp->time);
    
    //  marking of nodes switched on
    Gtm->update_nodes ();
    
    //  marking of DOFs switched on
    Gtm->update_dofs (Mp->time);
    
    /*
    fprintf (Out,"\n\n kontrola pole leso v case %le\n",Mp->time);
    for (j=0;j<Mt->ne;j++){
      fprintf (Out,"\n leso %4ld    %ld",j+1,Gtm->leso[j]);
    }
    
    fprintf (Out,"\n\n kontrola pole cn v case %le\n",Mp->time);
    for (j=0;j<Mt->nn;j++){
      fprintf (Out,"\n node %4ld    %4ld %4ld",j+1,Gtm->gnodes[j].cn[0],Gtm->gnodes[j].cn[1]);
    }
    */
    
    //  generation of new code numbers
    Ndofm = Gtm->codenum_generation (Out);
    nm=Ndofm;
    
    /*
    fprintf (Out,"\n\n kontrola pole cn v case %le po generovani kodovych cisel\n",Mp->time);
    for (j=0;j<Mt->nn;j++){
      fprintf (Out,"\n node %4ld    %4ld %4ld",j+1,Gtm->gnodes[j].cn[0],Gtm->gnodes[j].cn[1]);
    }
    fprintf (Out,"\n\n  NDOF %ld",n);
    fprintf (stdout,"  NDOF %ld",n);
    */
    
    /*
    fprintf (kon,"\n cas %le  vypocet",Mp->time);
    for (j=0;j<Gtm->nn;j++){
      fprintf (kon,"\n uzel %6ld",j+1);
      long ndofn=Gtm->gnodes[j].ndofn;
      for (long k=0;k<ndofn;k++){
	fprintf (kon," %ld",Gtm->gnodes[j].cn[k]);
      }
    }
    */
    
    if (nce!=0 || ncd!=0){
      //  assembling of displacement vector for new code numbers
      Mt->lhs_restore (r,nm);
      //  assembling of vector of previous nodal forces for new code numbers
      Mt->rhs_restore (fp,nm);
    }
    
    Gtm->node_mark (nm);
    
    Mm->updateipval();
    
    //  cleaning of the stiffness matrix
    //if (ncd>0){
    if (Smat != NULL){
      delete Smat;
      Smat=NULL;
    }    
    //}  
    
    
    /***************************************************/
    // TKO GRAPHICS
    /* if (nce || ncd){
       if (idcp > 0){
       print_close();
       }
       //compute_req_val (lcid);
       
       fprintf (stdout,"\n tisk do souboru");
       print_init(idcp, "wt");
       print_step(lcid, i, Mp->time, f);
       print_flush();
       idcp++;
       }
    // TKO GRAPHICS
    */
    /***************************************************/
    
    //  prescribed nodal forces
    mefel_right_hand_side (lcid,f,fl);
    
    //  computation of sums of force components in particular directions
    //  it is used in special creep and consolidation models
    Mb->comp_sum (f);

    //  forces caused by stress increments
    incr_internal_forces (lcid,fi);
    
    //fprintf (Out,"\n\n kontrola prave strany \n");
    
    //  right hand side assembling and storage of actual prescribed forces
    for (j=0;j<nm;j++){
      fb[j]=f[j]-fp[j]+fi[j];
    }
    
    //  allocation and assembling of the stiffness matrix
    stiffness_matrix (lcid);

    //  solution of K(r).v=F
    Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);
    //Smat->solve_system (Gtm,dr,fb);
    
    //fprintf (Out,"\n\n kontrola reseni \n");
    
    //  new displacement vector
    for (j=0;j<nm;j++){
      r[j]+=dr[j];
      //fprintf (Out,"\n dr %4ld  %10.5le      r %4ld  %10.5le",j,dr[j],j,r[j]);
    }

    //  computation of internal forces
    internal_forces (lcid,fi);
        
    norfa=ss(f,f,nm);
    for (j=0;j<nm;j++){
      fb[j] = fl[j] - fi[j];
    }
    //  norm of vector of unbalanced forces
    norf=ss(fb,fb,nm);
    if (norfa > 0.0)
      norf /= norfa;

    fprintf (stdout,"\n\n Norf before inner iteration loop norf = %e\n\n\n",norf);
    
    j = 0;
    if (norf<err){
      nsts++;      
    }
    else{
      // iteration of unbalanced forces caused by time independent models
      //  internal iteration loop
      fillm(0.0, lsm_a);
      fillv(0.0, lsm_r);
      for (j=0;j<ini;j++)
      {
        //  solution of K(r).v=F
	//Smat->solve_system (Gtm,dr,fb);
	Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);
	  
	for (k=0;k<nm;k++)
	  r[k]+=dr[k];
	  
	//  computation of internal forces
	internal_forces (lcid,fi);
	  
	//  vector of unbalanced forces
	for (k=0;k<nm;k++)
	  fb[k]=fl[k] - fi[k];
	  
	//  norm of vector of unbalanced forces
	norf=ss(fb,fb,nm);
	if (norfa > 0.0)
	  norf /= norfa;
	  
	if (Mespr==1)
//        if ((Mespr==1)&&(j%10==0)) // print each 10-th step
  	  fprintf (stdout,"\n Inner loop number j = %ld,     norf = %e \n",j,norf);
	  
	if (norf<err)  break;

        // divergence detection with help of least square method
        if (j > 1)
        {
          if (lsm_quad(lsm_a, lsm_r, lsm_l, double(j+1), norf, Mp->zero,1) > 0.0)
	  {
	    fprintf (stdout,"\n\n Divergence of inner iteation loop is detected. Inner loop is stopped.\n\n");
	    break;
	  }
	}
        else
          lsm_quad(lsm_a, lsm_r, lsm_l, double(j+1), norf, Mp->zero,0);
      }
    }
    //////  JK

    if (j==ini || norf>err)
      {
	
	//  backup is used for actual values
	for (j=0;j<nm;j++){
	  r[j]=lhsb[j];
	}
	
	//  reduction of the time increment because
	//  inner loop was not able to enforce equilibrium
	dt/=2.0;
	Mp->timecon.oldtime ();
	i--;
	
	fprintf (stdout,"\n\n time increment is reduced because the inner loop was not able to enforce equilibrium");
	if (dt<dtmin){
	  fprintf (stderr,"\n\n time increment is less than minimum time increment");
	  fprintf (stderr,"\n computation fails (file %s, line %d)\n",__FILE__,__LINE__);
          compute_req_val (lcid);
          print_step_forced(lcid, i, Mp->time, f);
          print_flush();
	  break;
	}
      }
    else
      {

	//  storage of displacements to nodes (possible changes in code numbers)
	//old:???????
	Mt->lhs_save (r,1);
	//Mt->lhs_save (dr,1);
	//  storage of prescribed forces to nodes (possible changes in code numbers)
	Mt->rhs_save (f);
	

	// equilibrium state was reached
	fprintf (stdout,"\n\n Last inner iteration step number j = %ld,     norf = %e \n\n",j,norf);

        //  actual total load vector becomes previous total load vector
        for (j=0;j<nm;j++){
          fp[j]=f[j];
        }
	
	dtdef = Mp->timecon.actualforwtimeincr ();
	
	if (nsts==2){
	  dt*=2.0;
	  nsts=0;
	  fprintf (stdout,"\n\n time increment is enlarged because no inner loop was neccessary in previous 3 steps");
	}
	if (dt>dtdef)
	  dt=dtdef;
	  
	Mm->updateipval();
	compute_req_val (lcid);
	print_step(lcid, i, Mp->time, fb);
	print_flush();
	
	if ((Mp->timecon.isitimptime ()==1) && (Mp->hdbcont.hdbtype))
	  mtsolver_save (r,fp,i,Mp->time,dt,nm);

	//backup of previous load
	for (j=0;j<nm;j++){
	  fp[j]=f[j];
	}  
      }
  }while(Mp->time<end_time);
  
  print_close ();
  
  //if (Mp->timecon.nit > 0)
  //fclose (aux);
  
  delete [] dr;  delete [] fi;  delete [] fb;  delete [] fp;  delete [] fl;  delete [] lhsb;
}


/**
   function solves problem of construction phases
   
   only one load case may be defined, one load case may contain
   several effects which are activated by time functions
   variable Mb->nlc must be equal to 1
   
   6.2.2006, JK
*/
void solve_prob_constr_phases_veryold ()
{
  long i,j,n,ncd,nce,lcid,idcp,ini;
  double prev_time,end_time,zero,*f,*fi,*fb,*fp,*r,*dr;
  double err;  
  char fname[1020];
  char *path;
  char *name;
  char *suffix;
  FILE *aux;
  
  //  load case id must be equal to zero
  lcid=0;
  
  //  definition of all output and auxiliary files
  Mp->filename_decomposition (Outdm->outfn,path,name,suffix);
  sprintf (fname,"%s%s.bac",path, name);

  delete [] path; delete [] name; delete [] suffix;

  if (Mp->timecon.nit > 0)
    aux = fopen (fname,"w");

  ini = 100;
  err = 1.0e-5;  
  //  number of rows of the matrix
  n = Ndofm;
  //  computer zero
  zero = Mp->zero;
  
  fp = new double [n];
  fb = new double [n];
  fi = new double [n];
  dr = new double [n];
  memset (fp,0,n*sizeof(double));
  memset (fb,0,n*sizeof(double));
  memset (fi,0,n*sizeof(double));
  memset (dr,0,n*sizeof(double));
  
  //  initialization phase
  r = Lsrs->give_lhs (lcid);
  f = Lsrs->give_rhs (lcid);
  
  //  initial time
  Mp->time=Mp->timecon.starttime ();
  prev_time=Mp->time;
  //  end time
  end_time = Mp->timecon.endtime ();
  
  //  allocation of backup arrays for state variables in integration points
  //Mm->alloc_backup ();
  
  //  provizorni umisteni, bude v mefelinit
  Gtm->comp_domain_sizes ();
  //Mb->alloc_sumcomp ();
  //  konec provizoria
  
  //Gtm->cn_clean ();
  
  nullv (fp,n);
  i=0;
  idcp = 0;
  
  //BEGIN OF GRAPHICS:
  //print_init(-1, "wt");
  //print_step(lcid, i, Mp->time, f);
  //print_flush();
  //print_close();
  //END OF GRAPHICS

  //FILE *kon;
  //kon = fopen ("kontrolavondr","w");
  //kon = fopen ("caskon","w");
  
  //  initiation of mechanical material models
  Mm->initmaterialmodels();
  
  
  Gtm->nm = new long [Gtm->nn];
  
  
  // ***************************
  //   main iteration loop  ****
  // ***************************
  do{

    //  indicator of strain computation
    //  it means, strains at integration points have not been computed
    Mp->strainstate=0;
    //  indicator of stress computation
    //  it means, stresses at integration points have not been computed
    Mp->stressstate=0;
    //  indicator of computation of other array
    //  it means, stresses at integration points have not been computed
    Mp->otherstate=0;

    
    fprintf (stdout,"\n time %e",Mp->time);

    //  searching for new elements
    //  only new elements are switched on
    //  former elements are temporarily switched off
    nce = Gtm->search_newelem (Mp->time,prev_time);
    
    if (nce!=0){
      //  attained displacements have to be saved on new elements
      //  otherwise stresses will not be correct
      Mt->initial_displ ();
    }
    
    //  searching for new DOFs
    //  only new DOFs are switched on
    //  former DOFs are temporarily switched off
    ncd = Gtm->search_newdofs (Mp->time,prev_time);

    /***************************************************/
    //////BEGIN OF GRAPHICS
    /*
    long graphics=0;
    if ((nce!=0 || ncd!=0) && graphics==1){
      //if (nce!=0 || ncd!=0){
      // ************************
      //  correction of graphic
      // ************************
      
      
      //fprintf (kon,"\n cas %le  grafika",Mp->time);
      //for (j=0;j<Gtm->nn;j++){
      //fprintf (kon,"\n uzel %6ld",j+1);
      //long ndofn=Gtm->gnodes[j].ndofn;
      //for (long k=0;k<ndofn;k++){
      //fprintf (kon," %ld",Gtm->gnodes[j].cn[k]);
      //}
      //}

      //  generation of new code numbers
      Ndofm = Gtm->gencodnum ();
      n=Ndofm;
      
      //  cleaning of the stiffness matrix
      if (Smat != NULL){
        delete Smat;
        Smat=NULL;
      }
      

      //fprintf (kon,"\n cas %le  grafika",Mp->time);
      //for (j=0;j<Gtm->nn;j++){
      //fprintf (kon,"\n uzel %6ld",j+1);
      //long ndofn=Gtm->gnodes[j].ndofn;
      //for (long k=0;k<ndofn;k++){
      //fprintf (kon," %ld",Gtm->gnodes[j].cn[k]);
      //}
      //}
      
      //  allocation and assembling of the stiffness matrix
      stiffness_matrix (lcid);
      
      //  nodal forces caused by attained displacements
      rhs_graphic (fb,lcid=0,n);
      
      //  solution of system of equations
      Smat->solve_system (dr,fb);
      
      //  storage of auxiliary displacements to nodes
      Mt->lhs_save (dr,2);
    }
    */
    //////END OF GRAPHICS
    /***************************************************/


    //  marking of elements switched on
    Gtm->update_elem (Mp->time);
    
    //  marking of nodes switched on
    Gtm->update_nodes ();
    
    //  marking of DOFs switched on
    Gtm->update_dofs (Mp->time);
    
    /*
    fprintf (Out,"\n\n kontrola pole leso v case %le\n",Mp->time);
    for (j=0;j<Mt->ne;j++){
      fprintf (Out,"\n leso %4ld    %ld",j+1,Gtm->leso[j]);
    }
    
    fprintf (Out,"\n\n kontrola pole cn v case %le\n",Mp->time);
    for (j=0;j<Mt->nn;j++){
      fprintf (Out,"\n node %4ld    %4ld %4ld",j+1,Gtm->gnodes[j].cn[0],Gtm->gnodes[j].cn[1]);
    }
    */

    //  generation of new code numbers
    //Ndofm = Gtm->codenum_generation (Out);
    n=Ndofm;
    
    /*
    fprintf (Out,"\n\n kontrola pole cn v case %le po generovani kodovych cisel\n",Mp->time);
    for (j=0;j<Mt->nn;j++){
      fprintf (Out,"\n node %4ld    %4ld %4ld",j+1,Gtm->gnodes[j].cn[0],Gtm->gnodes[j].cn[1]);
    }
    fprintf (Out,"\n\n  NDOF %ld",n);
    fprintf (stdout,"  NDOF %ld",n);
    */
    
    /*
    fprintf (kon,"\n cas %le  vypocet",Mp->time);
    for (j=0;j<Gtm->nn;j++){
      fprintf (kon,"\n uzel %6ld",j+1);
      long ndofn=Gtm->gnodes[j].ndofn;
      for (long k=0;k<ndofn;k++){
        fprintf (kon," %ld",Gtm->gnodes[j].cn[k]);
      }
    }
    */

    //if (nce!=0 || ncd!=0){
      //  assembling of displacement vector for new code numbers
      Mt->lhs_restore (r,n);
      //  assembling of vector of previous nodal forces for new code numbers
      Mt->rhs_restore (fp,n);
      //}

      Gtm->node_mark (n);

    Mm->updateipval();

    //  cleaning of the stiffness matrix
    //if (ncd>0){
    if (Smat != NULL){
      delete Smat;
      Smat=NULL;
    }    
    //}  


    /***************************************************/
    // TKO GRAPHICS
    /* if (nce || ncd){
       if (idcp > 0){
       print_close();
       }
       //compute_req_val (lcid);
       
       fprintf (stdout,"\n tisk do souboru");
       print_init(idcp, "wt");
       print_step(lcid, i, Mp->time, f);
       print_flush();
       idcp++;
       }
    // TKO GRAPHICS
    */
    /***************************************************/
    
    //  prescribed nodal forces
    mefel_right_hand_side (lcid,f);
    
    //  computation of sums of force components in particular directions
    //  it is used in special creep and consolidation models
    //Mb->comp_sum (f);

    //  forces caused by stress increments
    Mp->phase=1;
    internal_forces (lcid,fi);
    
    //fprintf (Out,"\n\n kontrola prave strany \n");
    
    //  right hand side assembling and storage of actual prescribed forces
    for (j=0;j<n;j++){
      //fprintf (Out,"\n f %4ld  %10.5le       fp %4ld  %10.5le    rozdil %10.5le",j,f[j],j,fp[j],f[j]-fp[j]);

      fb[j]=f[j]-fp[j]+fi[j];
      //fb[j]=f[j]-fp[j];
      fp[j]=f[j];
    }
    
    //  allocation and assembling of the stiffness matrix
    stiffness_matrix (lcid);

    //  solution of K(r).v=F
    Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);
    //Smat->solve_system (Gtm,dr,fb);
    
    //fprintf (Out,"\n\n kontrola reseni \n");
    
    //  new displacement vector
    for (j=0;j<n;j++){
      r[j]+=dr[j];
      //fprintf (Out,"\n dr %4ld  %10.5le      r %4ld  %10.5le",j,dr[j],j,r[j]);
    }

    //  storage of displacements to nodes (possible changes in code numbers)
    //old:???????
    Mt->lhs_save (r,1);
    //Mt->lhs_save (dr,1);
    //  storage of prescribed forces to nodes (possible changes in code numbers)
    Mt->rhs_save (f);
    
    
    //  computation of internal forces
    Mp->phase=2;
    internal_forces (lcid,fi);


    /***************************************************/
    
    //old
    // computation of values required for output    
    compute_req_val (lcid);
    
    //  time increment
    prev_time=Mp->time;
    Mp->time = Mp->timecon.newtime ();
    i++;
    
    if (Mp->timecon.isitimptime ()==1)
      mtsolver_save (r,fp,i,Mp->time,0.0,n);
    
    
    //fprintf (stdout,"\n tisk do souboru");
    //print_init(i, "wt");
    print_step(lcid, i, Mp->time, f);
    print_flush();
    
    /***************************************************/
    /***************************************************/
    
    /*
    //new:
    norfa=ss(f,f,n);
    for (j=0;j<n;j++){
      fb[j] = f[j] - fi[j];
    }
    //  norm of vector of unbalanced forces
    norf=ss(fb,fb,n);
    if (norfa > 0.0)
      norf /= norfa;
    
    if (norf<err){
      //  backup of nodal values because code numbers may be changed
      //new:
      Mt->lhs_save (r,1);
      Mt->rhs_save (f);
      
      // computation of values required for output    
      compute_req_val (lcid);
      //  time increment
      prev_time=Mp->time;
      Mp->time = Mp->timecon.newtime ();
      Mp->timecon.take_values (Mp->timecon);
      
      if (Mp->timecon.isitimptime ()==1){
        mtsolver_save (aux,r,fp,i,Mp->time,n,0,0);
      }
      
      print_step(lcid, i, Mp->time, f);
      print_flush();
      i++;      
    }
    
    // iteration of unbalanced forces caused by time independent models
    //  internal iteration loop
    else{
      for (j=0;j<ini;j++)
        {
          //  solution of K(r).v=F
          Smat->solve_system (dr,fb);
          
          for (k=0;k<n;k++)
            r[k]+=dr[k];
          
          //  computation of internal forces
          internal_forces (lcid,fi);
          
          //  vector of unbalanced forces
          for (k=0;k<n;k++)
            fb[k]=f[k]-fi[k];
          
          //  norm of vector of unbalanced forces
          norf=ss(fb,fb,n);
          if (norfa > 0.0)
            norf /= norfa;
          
          if (Mespr==1) fprintf (stdout,"\n Time increment %lf   inner loop j %ld     norf=%e",Mp->time,j,norf);
          if (norf<err)  break;
        }
      if (j==ini || norf>err)
        {
          if (Mespr==1) fprintf (stdout,"\n Iteration process does not converge req. err = %e, reached err = %e", err,norf);
          
          //  backup of nodal values because code numbers may be changed
          //new:
          Mt->lhs_save (r,1);
          Mt->rhs_save (f);
          
          // equilibrium state was reached not - added by TKr
          compute_req_val (lcid);
          //  time increment
          prev_time = Mp->time;
          Mp->time = Mp->timecon.newtime ();
          Mp->timecon.take_values (Mp->timecon);
          
          if (Mp->timecon.isitimptime ()==1)
            mtsolver_save (aux,r,fp,i,Mp->time,n,0,0);
          print_step(lcid, i, Mp->time, f);
          print_flush();
          i++;
        }
      else
        {
          //  backup of nodal values because code numbers may be changed
          //new:
          Mt->lhs_save (r,1);
          Mt->rhs_save (f);
          
          // equilibrium state was reached
          compute_req_val (lcid);
          //  time increment
          prev_time = Mp->time;
          Mp->time = Mp->timecon.newtime ();
          Mp->timecon.take_values (Mp->timecon);
          
          if (Mp->timecon.isitimptime ()==1)
            mtsolver_save (aux,r,fp,i,Mp->time,n,0,0);
          print_step(lcid, i, Mp->time, f);
          print_flush();
          i++;
        }
    }
    */

  }while(Mp->time<end_time);
  
  print_close ();
  
  //fclose (kon);
  
  
  if (Mp->timecon.nit > 0)
    fclose (aux);
  
  delete [] dr;  delete [] fi;  delete [] fb;  delete [] f, delete [] fp;
}

/*




    if (nce!=0 || ncd!=0)
    
    //  toto umisteni je jen pokusne
    Mm->updateipval();
    
    fprintf (stdout,"\n\n cas %le",Mp->time);
    fprintf (stdout,"\n pocet zmen na prvcich   %ld",nce);

    Gtm->print_time_functions (kon,Mp->time);



    if (nce>0){
      //  nce>0 means changed elements
      
      //  attained displacements have to be saved at new elements
      //Mt->initial_displ ();
      
      
      fprintf (stdout,"\n\n pocitame korekci posunu na zapnute casti v case %le",Mp->time);
      
      
      //  searching for new nodes
      ncd = Gtm->search_newdofs (Mp->time,prev_time);
      
      
      
      
      
      
      //  definition of DOFs
      Ndofm = Gtm->gencodnum ();
      n=Ndofm;
      if (Smat != NULL){
        delete Smat;
        Smat=NULL;
      }
      

      rhs_graphic (fb,lcid=0,n);

      //  assembling of tangent stiffness matrix
      stiffness_matrix (lcid);
      
      //  solution of K(r).v=F
      Smat->solve_system (dr,fb);
      
      
      //  backup of nodal values because code numbers may be changed
      Mt->lhs_save (dr,2);
      
      
      //  array leso is restored
      //  it was changed in function Gtm->search_newelem (time)
      //Gtm->restore_leso ();
      //Gtm->cn_restore ();
      
      
      Gtm->update_dofs (Mp->time);
    }








    ncd = Gtm->search_newdofs (Mp->time,prev_time);
    Gtm->update_dofs (Mp->time);
    Ndofm = Gtm->gencodnum ();
    n=Ndofm;
    
    //  update of nodes, DOFs and elements
    //Gtm->update_mesh (Mp->time,prev_time,ncn,nce);
    
    //  set up of correct values in array leso
    Gtm->update_elemstat (Mp->time);
    
    
    Gtm->print_time_functions (kon,Mp->time);
    
    fprintf (stdout,"\n cas %le  zmeny v DOF %ld    zmeny v prvcich %ld",Mp->time,ncd,nce);
    
    
    
    //  definition of DOFs
    if (ncd>0){
      //Ndofm = Gtm->gencodnum ();
      //n=Ndofm;

      if (Smat != NULL){
        delete Smat;
        Smat=NULL;
      }

      //  assembling of lhs array for new code numbers
      Mt->lhs_restore (r,n);
      Mt->rhs_restore (fp,n);
    }
    


    long k;
    fprintf (kon,"\n\n\n\n novy zapis (cas %le)\n\n\n\n",Mp->time);
    for (j=0;j<Mt->nn;j++){
      if (Gtm->lnso[j]==1){
        fprintf (kon,"%5ld     ",j+1);
        for (k=0;k<Mt->give_ndofn (j);k++){
          fprintf (kon,"%5ld   ",Gtm->gnodes[j].cn[k]);
        }
        fprintf (kon,"  %le   %le   %le\n",Gtm->gnodes[j].x,Gtm->gnodes[j].y,Gtm->gnodes[j].z);
      }
    }

    //fclose (kon);
    //abort ();

    
    fprintf (stdout,"\n pocet neznamych  %ld",n);
    
    if (nce || ncd){
      if (idcp > 0){
        print_close();
      }
      compute_req_val (lcid);
      
      fprintf (stdout,"\n tisk do souboru");
      print_init(idcp, "wt");
      print_step(lcid, i, Mp->time, f);
      print_flush();
      idcp++;
    }
    
    //fprintf (stdout,"\n time %e,      NDOF %ld\n",Mp->time,Ndofm);
    
    //for (j=0;j<29;j++){
    //fprintf (stdout,"%ld %ld\n",j,Gtm->gf[j].getval_long (Mp->time));
    //}
    


    //  assignment of initial displacements
    //  it works for problem with growing number of elements
    //  it does not work generally for problems with increasing and decreasing number of elements
    if (nce>0){
      Mt->initial_displ ();
    }


    fprintf (kon,"\n\n cas  %lf \n\n",Mp->time);
    fprintf (kon,"\n\n kontrola kodovych cisel \n\n");
    long k,ndofn;
    for (j=0;j<Mt->nn;j++){
      fprintf (kon,"\n uzel %ld",j+1);
      ndofn=Gtm->gnodes[j].ndofn;
      for (k=0;k<ndofn;k++){
        fprintf (kon,"  %ld",Gtm->gnodes[j].cn[k]);
      }
    }
    fprintf (kon,"\n\n");
    
    fprintf (kon,"\n\n\n kontrola pocatecnich posunu na prvcich \n");
    for (j=0;j<Gtm->ne;j++){
      fprintf (kon,"\n element %ld\n",j+1);
      long ndofe=Gtm->give_ndofe (j);
      for (k=0;k<ndofe;k++){
        fprintf (kon,"  %lf",Mt->elements[j].initdispl[k]);
      }
    }
    fprintf (kon,"\n\n");
    

    
    Mm->updateipval();
    //fprintf (stdout,"\n time %e,      NDOF %ld",Mp->time,Ndofm);
    //    fprintf (Out,"\n\n\n\n time %e",Mp->time);
    
    //  prescribed nodal forces
    mefel_right_hand_side (lcid,f);

    // initialization of material models, initial temperatures are
    // computed during right hand side assembling -> right hand side
    // have to be reassembled due to correct temprstrains
    if (i==0)
    {
    Mm->initmaterialmodels();
      mefel_right_hand_side (lcid,f);
    }
    
    //Mb->comp_sum (f);
    

    //  unbalanced forces
    Mp->phase=1;
    //Mm->est=eigstrain;
    internal_forces (lcid,fi);
    
    fprintf (kon,"\n\n\n kontrola velkych vektoru \n");
    for (j=0;j<n;j++){
      fprintf (kon,"\n %4ld  f %12.8lf   fp %12.8lf   fi %12.8lf   fb %12.8lf",j,f[j],fp[j],fi[j],f[j]-fp[j]);
    }
    fprintf (kon,"\n\n");

    //  right hand side assembling
    for (j=0;j<n;j++){
      //fb[j]=f[j]-fp[j]+fi[j];
      fb[j]=f[j]-fp[j];
      fp[j]=f[j];
    }
    
    //fprintf (Out,"\n\n kontrola prave strany \n");
    //for (i=0;i<n;i++){
    //fprintf (Out,"\n fb %5ld   %le",i,fb[i]);
    //}
    
    
    //  assembling of tangent stiffness matrix
    stiffness_matrix (lcid);
    
    //  solution of K(r).v=F
    Smat->solve_system (dr,fb);
    
    //  new displacement vector
    for (j=0;j<n;j++){
      r[j]+=dr[j];
    }
    //  backup of nodal values because code numbers may be changed
    Mt->lhs_save (r,1);
    Mt->rhs_save (f);
    
    fprintf (kon,"\n\n kontrola reseni \n\n");
    for (j=0;j<Mt->nn;j++){
      fprintf (kon,"\n uzel %ld",j+1);
      ndofn=Mt->give_ndofn (j);
      long cn;
      for (k=0;k<ndofn;k++){
        cn=Mt->give_dof (j,k);
        if (cn==0)
          fprintf (kon,"  %lf",0.0);
        else
          fprintf (kon,"  %lf",dr[cn-1]);
      }
      fprintf (kon,"   ");
      for (k=0;k<ndofn;k++){
        cn=Mt->give_dof (j,k);
        if (cn==0)
          fprintf (kon,"  %lf",0.0);
        else
          fprintf (kon,"  %lf",r[cn-1]);
      }

    }
    
    
    
    
    

    long kj,ndofn;
    double *jk;
    fprintf (Out,"\n\n\n cas %le\n",Mp->time);
    for (j=0;j<Mt->nn;j++){
      ndofn = Mt->give_ndofn (j);
      jk = new double [ndofn];
      noddispl (0,jk,j);
      fprintf (Out,"\n\n uzel  %5ld",j+1);
      for (kj=0;kj<ndofn;kj++){
        fprintf (Out,"\n slozka %5ld   d %le   id %le   d+id %le",kj,jk[kj],Mt->nodes[j].nvgraph[kj],jk[kj]+Mt->nodes[j].nvgraph[kj]);
      }
      delete jk;
    }

    
    
    //  computation of internal forces
    Mp->phase=2;
    internal_forces (lcid,fi);

    // computation of values required for output    
    compute_req_val (lcid);

    //  time increment
    prev_time=Mp->time;
    Mp->time = Mp->timecon.newtime ();
    i++;
    
    
    if (Mp->timecon.isitimptime ()==1){
      mtsolver_save (aux,r,fp,i,Mp->time,n,0,0);
    }

    
    fprintf (stdout,"\n tisk do souboru");
    print_init(i, "wt");
    print_step(lcid, i, Mp->time, f);
    print_flush();
    //print_close();
    
  }while(Mp->time<end_time);
  
  print_close ();
  
  fclose (kon);


  if (Mp->timecon.nit > 0)
    fclose (aux);

  delete [] dr;  delete [] fi;  delete [] fb;  delete [] f, delete [] fp;
}

*/
