#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "genfile.h"
#include "global.h"
#include "intpoints.h"
#include "creep.h"
#include "elastisomat.h"
#include "creep_b3.h"
#include "creep_rspec.h"
#include "creepb.h"
#include "creep_dpl.h"


/**
   function returns actual stiffness matrix

   @param d   - stiffness %matrix
   @param ipp - integration point
   @param im  - index of material type for given ip
   @param ido - index of internal variables for given material in the ipp other array
   
   TKr, 07/08/2008 - revised
*/
void creep_matstiff (matrix &d, long ipp,long im,long ido)
{
  double q;
  q = 1.0;
  // asymptotic elastic stiffness matrix
  Mm->elmatstiff (d,ipp);  
  // actual stiffness  coefficient (modulus)
  q = creep_matstiffchange(ipp,im,ido);
  
  // actual stiffness matrix
  cmulm(q,d);
}


/**
   function returns actual stiffness (modulus)

   @param ipp - integration point
   @param im  - index of material type for given ip
   @param ido - index of internal variables for given material in the ipp other array
   
   TKr, 07/08/2008 - revised
*/
double creep_matstiffchange (long ipp,long im,long ido)
{
  long i;
  long n_ret_times,imat,napproxtime;
  double tb_age_dt,tb_age,tl_age,th_age,dt,timeMax,q,delYi;
  double e0,et,qq;
  double inv_v,q1,c_const;
  vector ret_times,emu;

  //number of retardation times
  n_ret_times = creep_number_rettimes (ipp,im);
  
  allocv (n_ret_times,ret_times);
  fillv(0.0, ret_times);
  allocv (n_ret_times,emu);
  fillv(0.0, emu);
  
  imat=Mm->ip[ipp].idm[Mm->ip[ipp].gemid()];
  
  //asympotic elastic modulus
  e0=Mm->eliso[imat].e;
  
  switch (Mm->ip[ipp].tm[im]){
  case creepb3:{
    i=Mm->ip[ipp].idm[im];
    //times -> ages [days]
    //actuall times in days
    Mm->crb3[i].give_ages (tb_age_dt,tb_age,tl_age,th_age,dt,timeMax,napproxtime,ipp);
    //retardation times from ip
    Mm->crb3[i].give_rettimes(ret_times,n_ret_times,ipp);
    //coefficients of Dirichlet series from ip (Kelvin chain model)
    Mm->crb3[i].give_emu_eqother(emu,n_ret_times,ipp,ido);
    //ageing function 1/v(t)    
    inv_v = 0.0;
    //inital compliance
    q1 = Mm->crb3[i].give_q1();
    // const
    c_const = 0.0;
    break;
  }
  case creeprs:{
    i=Mm->ip[ipp].idm[im];
    //times -> ages [days]
    //actuall times in days
    Mm->crrs[i].give_ages (tb_age_dt,tb_age,tl_age,th_age,dt,timeMax,napproxtime,ipp);
    //retardation times from ip
    Mm->crrs[i].give_rettimes(ret_times,n_ret_times,ipp);
    //coefficients of Dirichlet series from ip (Kelvin chain model)
    Mm->crrs[i].give_emu_eqother(emu,n_ret_times,ipp,ido);
    //ageing function 1/v(t)    
   inv_v = Mm->crrs[i].give_inv_v(tl_age);//aritmetical mid_interval value
    //inital compliance
    q1 = Mm->crrs[i].give_q1();
    // const
    c_const = Mm->crrs[i].give_C_const();
    break;
  }
  case creepdpl:{
    i=Mm->ip[ipp].idm[im];
    //times -> ages [days]
    //actuall times in days
    Mm->crdpl[i].give_ages (tb_age_dt,tb_age,tl_age,th_age,dt,timeMax,napproxtime,ipp);
    //retardation times from ip
    Mm->crdpl[i].give_rettimes(ret_times,n_ret_times,ipp);
    //coefficients of Dirichlet series from ip (Kelvin chain model)
    Mm->crdpl[i].give_emu_eqother(emu,n_ret_times,ipp,ido);
    //ageing function 1/v(t)    
    inv_v = 0.0;
    //inital compliance
    q1 = Mm->crdpl[i].give_q1();
    // const
    c_const = 0.0;
    break;
  }
  default:{
    print_err("\n Unknown material type required in ", __FILE__, __LINE__, __func__);
  }
  }
  
  //Computing of actual stiffness modulus from dirichlet series
  et=0.;
  q=2./3.;
  //Kelvin chain model
  for (i=0;i<n_ret_times;i++){
    if(tb_age<0.0001)
      delYi=pow((0.0001+dt)/ret_times[i],q)-pow(0.0001/ret_times[i],q);
    else 
      delYi=pow((tb_age+dt)/ret_times[i],q)-pow(tb_age/ret_times[i],q);
    
    et=et+(delYi-1.+exp(-delYi))/delYi/emu[i];
  }

  if((Mm->ip[ipp].tm[im]) == creeprs){
    et = et + c_const;
    et = et * inv_v;
  }
  //added q1 term (inital compliance)
  et = et + q1;
  
  //Do not delete this part. This is here for testing creep models.
  /* double dd=0.0;
     long j;
     if (Mm->ip[ipp].tm[im] == creeprs){
     i=Mm->ip[ipp].idm[im];
     for(j=1;j<1000;j++){
     //et = Mm->crrs[i].give_J_E_mu(emu,th_age,tb_age,tb_age+dd,ipp,ido);
     //et = (et-q1)*inv_v;
     et = Mm->crb3[0].b3_law (th_age,tb_age,tb_age+dd,ipp,ido);
     et = et + q1;
     fprintf (Out,"%e   %e\n",tb_age+dd,-et);
     dd = dd + 1.0;
     }
     }
     fclose(Out);
  */
  
  //actual stiffness
  if(et>1.e-50)
    et=1./et;
  else{
    fprintf (Out,"\n\n Stiffness modulus is zero\n");
    fprintf (stderr,"\n\n Stiffness modulus is zero\n");
    abort();
  }

  destrv(ret_times);
  destrv(emu);
  
  //  actual material modulus
  qq=et/e0;

  //qq = 1.0;//tady je pro testovani nastaven elasticky modul pruznosti??!!

  return(qq);
}


/** 
   function returns actual Young's modulus
    
   @param ipp - integration point
   @param im  - index of material type for given ip
   @param ido - index of internal variables for given material in the ipp other array
   
   TKr, 07/08/2008 - revised
*/ 
double creep_give_actual_ym (long ipp,long im,long ido) 
{ 
  long i,imat;
  double q,e_0,e_t;

  imat=Mm->ip[ipp].idm[Mm->ip[ipp].gemid()];
  //creep coefficient
  q = creep_matstiffchange (ipp,im,ido);
  //asympotic elastic modulus 
  e_0=Mm->eliso[imat].e;
  e_t = q*e_0;
  
  switch (Mm->ip[ipp].tm[im]){
  case creepb3:{
    i=Mm->ip[ipp].idm[im];
    Mm->crb3[i].store_ym_eqother(e_t,ipp,ido);
    break;
  }
  case creeprs:{
    i=Mm->ip[ipp].idm[im];
    Mm->crrs[i].store_ym_eqother(e_t,ipp,ido);
    break;
  }
  case creepdpl:{
    i=Mm->ip[ipp].idm[im];
    Mm->crdpl[i].store_ym_eqother(e_t,ipp,ido);
    break;
  }
  default:{
    print_err("\n Unknown material type required in ", __FILE__, __LINE__, __func__);
  }
  }
  return (e_t);
} 


/** 
   function returns inital Young's modulus    

   @param ipp - integration point
   @param im  - index of material type for given ip
   @param ido - index of internal variables for given material in the ipp other array
   
   TKr, 07/08/2008 - revised
*/ 
double creep_give_inital_ym (long ipp,long im,long ido) 
{ 
  long i,imat;
  double q,e_0,e_t;

  imat=Mm->ip[ipp].idm[Mm->ip[ipp].gemid()];
  //creep coefficient
  q = creep_matstiffchange (ipp,im,ido);
  //asympotic elastic modulus 
  e_0=Mm->eliso[imat].e;
  e_t = q*e_0;
  
  switch (Mm->ip[ipp].tm[im]){
  case creepb3:{
    i=Mm->ip[ipp].idm[im];
    Mm->crb3[i].store_ym_eqother(e_t,ipp,ido);
    break;
  }
  case creeprs:{
    i=Mm->ip[ipp].idm[im];
    Mm->crrs[i].store_ym_eqother(e_t,ipp,ido);
    break;
  }
  case creepdpl:{
    i=Mm->ip[ipp].idm[im];
    Mm->crdpl[i].store_ym_eqother(e_t,ipp,ido);
    break;
  }
  default:{
    print_err("\n Unknown material type required in ", __FILE__, __LINE__, __func__);
  }
  }
  return (e_t);
} 


/** 
    function returns actual tensile strenght
    
    @param ipp - integration point
    @param im  - index of material type for given ip
    @param ido - index of internal variables for given material in the ipp other array
    
    TKr, 07/08/2008 - revised
*/ 
double creep_give_actual_ft (long ipp,long im,long ido) 
{ 
  long i;
  double fft;

  switch (Mm->ip[ipp].tm[im]){
  case creepb3:{
    i=Mm->ip[ipp].idm[im];
    fft = Mm->crb3[i].creep_give_actual_ft (ipp,im,ido);
    break;
  }
  case creeprs:{
    i=Mm->ip[ipp].idm[im];
    fft = Mm->crrs[i].creep_give_actual_ft (ipp,im,ido);
    break;
  }
  case creepdpl:{
    i=Mm->ip[ipp].idm[im];
    fft = Mm->crdpl[i].creep_give_actual_ft (ipp,im,ido);
    break;
  }
  default:{
    print_err("\n Unknown material type required in ", __FILE__, __LINE__, __func__);
  }
  }
  
  return (fft);
} 


/** 
   function returns actual compression strenght

   @param ipp - integration point
   @param im  - index of material type for given ip
   @param ido - index of internal variables for given material in the ipp other array
   
   TKr, 07/08/2008 - revised
*/ 
double creep_give_actual_fc (long ipp,long im,long ido) 
{ 
  long imat;
  double q,e_0,e_t,ffc;
  
  imat=Mm->ip[ipp].idm[Mm->ip[ipp].gemid()];
  //creep coefficient
  q = creep_matstiffchange (ipp,im,ido);
  //asympotic elastic modulus 
  e_0=Mm->eliso[imat].e;
  e_t = q*e_0;
  
  //not finished??!!
  ffc = e_t/1000.0;

  return (ffc);
} 


/**
   function initializes creep material model and retardation coefficient (stiffnesses of Kelvin chain units)

   @param ipp - integration point
   @param im  - index of material type for given ip
   @param ido - index of internal variables for given material in the ipp other array
   
   TKr, 07/08/2008 - revised
*/
void creep_initmaterialmodel(long ipp,long im,long ido)
{
  long i,j,k;
  long n_ret_times,napproxtime;
  double t_age,tb_age_dt,tb_age,tl_age,th_age,dt,timeMax,q,delYi,delYj,m0,m;
  double jt;
  vector bpom,ret_times,emu;
  matrix apom,cpom;
  
  //number of retardation times
  n_ret_times = creep_number_rettimes (ipp,im);
  
  allocv(n_ret_times,bpom);
  fillv(0.0, bpom);
  allocm(n_ret_times, n_ret_times, apom);
  fillm(0.0, apom);
  allocm(n_ret_times, n_ret_times, cpom);
  fillm(0.0, cpom);
  allocv (n_ret_times,ret_times);
  fillv(0.0, ret_times);
  allocv (n_ret_times,emu);
  fillv(0.0, emu);  
  jt=0.0;
  
  switch (Mm->ip[ipp].tm[im]){
  case creepb3:{
    i=Mm->ip[ipp].idm[im];    
    //time initialization
    Mm->crb3[i].previoustime = Mp->time;
    //compute initial values
    Mm->crb3[i].initvalues(ipp, ido);
    //times -> ages [days]
    //actuall times in days
    Mm->crb3[i].compute_ages (ipp,ido);
    Mm->crb3[i].give_ages (tb_age_dt,tb_age,tl_age,th_age,dt,timeMax,napproxtime,ipp);
    //ret. times from ip
    Mm->crb3[i].give_rettimes(ret_times,n_ret_times,ipp);
    q=2./3.;
    break;
  }
  case creeprs:{
    i=Mm->ip[ipp].idm[im];    
    //time initialization
    Mm->crrs[i].previoustime = Mp->time;
    //compute initial values
    Mm->crrs[i].initvalues(ipp, ido);
    //times -> ages [days]
    //actuall times in days
    Mm->crrs[i].compute_ages (ipp,ido);
    Mm->crrs[i].give_ages (tb_age_dt,tb_age,tl_age,th_age,dt,timeMax,napproxtime,ipp);
    //ret. times from ip
    Mm->crrs[i].give_rettimes(ret_times,n_ret_times,ipp);
    q=2./3.;

    break;
  }
  case creepdpl:{
    i=Mm->ip[ipp].idm[im];    
    //time initialization
    Mm->crdpl[i].previoustime = Mp->time;
    //times -> ages [days]
    //actuall times in days
    Mm->crdpl[i].compute_ages (ipp,ido);
    Mm->crdpl[i].give_ages (tb_age_dt,tb_age,tl_age,th_age,dt,timeMax,napproxtime,ipp);
    //ret. times from ip
    Mm->crdpl[i].give_rettimes(ret_times,n_ret_times,ipp);
    q=2./3.;
    break;
  }
  default:{
    print_err("\n Unknown material type required in ", __FILE__, __LINE__, __func__);
  }
  }

  //Dirichlet series
  if (Mm->ip[ipp].tm[im] != creeprs){
    t_age=tl_age+0.0001;
    
    m0 = log10(tl_age+0.0001);
    m = log10(2.0*timeMax);
    k=0;
    
    //computing actual stiffness from compliance function
    //coverting creep function into Dirichlet-Prony series
    while (t_age <= 2.0*timeMax){
      
      //creep function
      switch (Mm->ip[ipp].tm[im]){
      case creepb3:{
	i=Mm->ip[ipp].idm[im];
	//actuall compliance function
	jt = Mm->crb3[i].b3_law (th_age,tl_age,t_age,ipp,ido);
	break;
      }
      case creepdpl:{
	i=Mm->ip[ipp].idm[im];
	//actuall compliance function
	jt = Mm->crdpl[i].double_power_law (tl_age,t_age,ipp,ido);
	break;
      }
      default:{
	print_err("\n Unknown material type required in ", __FILE__, __LINE__, __func__);
      }
      }
      
      for (i=0;i<n_ret_times;i++){
	delYi=pow((tl_age/ret_times[i]),q)-pow((t_age/ret_times[i]),q);
	bpom[i]=bpom[i]+(1.-exp(delYi))*jt;
	for (j=0;j<n_ret_times;j++){
	  delYj= pow((tl_age/ret_times[j]),q)-pow((t_age/ret_times[j]),q);
	  apom[i][j]=apom[i][j]+(1.-exp(delYi))*(1.-exp(delYj));
	}
      }
      k=k+1;

      t_age=pow(10.,m0+(m-m0)/napproxtime*k);
    }
    
    invm(apom,cpom,1.0e-20);
    
    //computing coefficients of Dirichlet series
    for (i=0;i<n_ret_times;i++){    
      for (j=0;j<n_ret_times;j++)
	emu[i]=emu[i]+cpom[i][j]*bpom[j];
      
      emu[i]=1./emu[i];
    }
  }
  
  //Continuous retardation spectrum
  if (Mm->ip[ipp].tm[im] == creeprs){
    jt = Mm->crrs[i].give_J_E_mu(emu,th_age,tb_age,tb_age_dt,ipp,ido);
  }
  
  switch (Mm->ip[ipp].tm[im]){
  case creepb3:{
    i=Mm->ip[ipp].idm[im];
    //storing stiffness E_mu of Kelvin chains into itegration point
    Mm->crb3[i].store_emu_eqother(emu,n_ret_times,ipp,ido);
    break;
  }
  case creeprs:{
    i=Mm->ip[ipp].idm[im];
    //storing stiffness E_mu of Kelvin chains into itegration point
    Mm->crrs[i].store_emu_eqother(emu,n_ret_times,ipp,ido);
    break;
  }
  case creepdpl:{
    i=Mm->ip[ipp].idm[im];
    //storing stiffness E_mu of Kelvin chains into itegration point
    Mm->crdpl[i].store_emu_eqother(emu,n_ret_times,ipp,ido);
    break;
  }
  default:{
    print_err("\n Unknown material type required in ", __FILE__, __LINE__, __func__);
  }
  }
  
  destrm(apom);
  destrm(cpom);
  destrv(bpom);
  destrv(ret_times);
  destrv(emu);
}


/**
   function updates creep material model and retardation coefficient (stiffnesses of Kelvin chain units)
    
   @param ipp - integration point
   @param im  - index of material type for given ip
   @param ido - index of internal variables for given material in the ipp other array
   
   TKr, 07/08/2008 - revised
*/
void creep_updateval(long ipp,long im,long ido)
{
  long i,j,k,imat;
  long nc,n_ret_times,napproxtime;
  double mu;
  double t_age,tb_age_dt,tb_age,tl_age,th_age,dt,timeMax,q,delYi,delYj,m0,m;
  double jt,inv_v,q4,nonlin_func,dj;
  vector bpom,ret_times,emu;
  vector deps_cr,deps_sh,deps_ss,eps,deps_help;
  vector sigma,sigmao,dsigma;
  matrix apom,cpom;
  matrix screep,c;
  strastrestate ss;
  
  //  number of stres components on element
  nc=Mm->ip[ipp].ncompstr;
  //number of retardation times
  n_ret_times = creep_number_rettimes (ipp,im);
  //  stress/strain state
  //planestress=10,planestrain=11,plate=15,axisymm=20,shell=25,spacestress=30
  ss=Mm->ip[ipp].ssst;  

  imat=Mm->ip[ipp].idm[Mm->ip[ipp].gemid()];
  //Poisons ratio
  mu=Mm->give_actual_nu(ipp);

  jt=0.0;

  allocv(n_ret_times,bpom);
  fillv(0.0, bpom);
  allocm(n_ret_times, n_ret_times, apom);
  fillm(0.0, apom);
  allocm(n_ret_times, n_ret_times, cpom);
  fillm(0.0, cpom);
  allocv (n_ret_times,ret_times);
  fillv(0.0, ret_times);
  allocv (n_ret_times,emu);
  fillv(0.0, emu);
  
  allocm(nc, n_ret_times,screep);
  fillm(0.0,screep);
  allocv(nc,deps_cr);
  fillv(0.0,deps_cr);
  allocv(nc,deps_sh);
  fillv(0.0,deps_sh);
  allocv(nc,deps_ss);
  fillv(0.0,deps_ss);
  allocv(nc,eps);
  fillv(0.0,eps);

  allocv(nc,sigma);
  fillv(0.0,sigma);
  allocv(nc,sigmao);
  fillv(0.0,sigmao);
  allocv(nc,dsigma);
  fillv(0.0,dsigma);

  //storage of actual correct values
  switch (Mm->ip[ipp].tm[im]){
  case creepb3:{
    i=Mm->ip[ipp].idm[im];
    //actual time
    Mm->crb3[i].previoustime = Mp->time;
    // stores total strains into eqother
    for (j=0; j<nc; j++)
      eps[j]=Mm->ip[ipp].strain[j];
    Mm->crb3[i].store_strains_eqother(eps,ipp,ido);

    //update of material model
    Mm->crb3[i].updatevalues(ipp, ido);
    //times -> ages [days]
    //actuall times in days
    Mm->crb3[i].give_ages (tb_age_dt,tb_age,tl_age,th_age,dt,timeMax,napproxtime,ipp);
    //ret. times from ip
    Mm->crb3[i].give_rettimes(ret_times,n_ret_times,ipp);
    q=2./3.;
    break;
  }
  case creeprs:{
    i=Mm->ip[ipp].idm[im];
    //actual time
    Mm->crrs[i].previoustime = Mp->time;
    // stores total strains into eqother
    for (j=0; j<nc; j++)
      eps[j]=Mm->ip[ipp].strain[j];
    Mm->crrs[i].store_strains_eqother(eps,ipp,ido);

    //update of material model
    Mm->crrs[i].updatevalues(ipp, ido);
    //times -> ages [days]
    //actuall times in days
    Mm->crrs[i].give_ages (tb_age_dt,tb_age,tl_age,th_age,dt,timeMax,napproxtime,ipp);
    //ret. times from ip
    Mm->crrs[i].give_rettimes(ret_times,n_ret_times,ipp);
    q=2./3.;
    break;
  }
  case creepdpl:{
    i=Mm->ip[ipp].idm[im];
    //actual time
    Mm->crdpl[i].previoustime = Mp->time;
    // stores total strains into eqother
    for (j=0; j<nc; j++)
      eps[j]=Mm->ip[ipp].strain[j];
    Mm->crdpl[i].store_strains_eqother(eps,ipp,ido);
    
    //times -> ages [days]
    //actuall times in days
    Mm->crdpl[i].give_ages (tb_age_dt,tb_age,tl_age,th_age,dt,timeMax,napproxtime,ipp);
    //ret. times from ip
    Mm->crdpl[i].give_rettimes(ret_times,n_ret_times,ipp);
    q=2./3.;
    break;
  }
  default:{
    print_err("\n Unknown material type required in ", __FILE__, __LINE__, __func__);
  }
  }

  //computing of new retardation coefficients
  //Classical Dirichlet series
  if (Mm->ip[ipp].tm[im] != creeprs){
    t_age=tl_age+0.0001;
    
    m0 = log10(tl_age+0.0001);
    m = log10(2.0*timeMax);
    k=0;
    
    //computing actual stiffness from compliance function
    //coverting creep function into Dirichlet-Prony series
    while (t_age <= 2.0*timeMax){
      
      //creep function
      switch (Mm->ip[ipp].tm[im]){
      case creepb3:{
	i=Mm->ip[ipp].idm[im];
	//actuall compliance function
	jt = Mm->crb3[i].b3_law (th_age,tl_age,t_age,ipp,ido);
	break;
      }
      case creepdpl:{
	i=Mm->ip[ipp].idm[im];
	//actuall compliance function
	jt = Mm->crdpl[i].double_power_law (tl_age,t_age,ipp,ido);
	break;
      }
      default:{
	print_err("\n Unknown material type required in ", __FILE__, __LINE__, __func__);
      }
      }
      
      for (i=0;i<n_ret_times;i++){
	delYi=pow((tl_age/ret_times[i]),q)-pow((t_age/ret_times[i]),q);
	bpom[i]=bpom[i]+(1.-exp(delYi))*jt;
	for (j=0;j<n_ret_times;j++){
	  delYj= pow((tl_age/ret_times[j]),q)-pow((t_age/ret_times[j]),q);
	  apom[i][j]=apom[i][j]+(1.-exp(delYi))*(1.-exp(delYj));
	}
      }
      k=k+1;
      
      t_age=pow(10.,m0+(m-m0)/napproxtime*k);
    }
    
    invm(apom,cpom,1.0e-20);
    
    //computing of new coefficients of Dirichlet series
    for (i=0;i<n_ret_times;i++){    
      for (j=0;j<n_ret_times;j++)
	emu[i]=emu[i]+cpom[i][j]*bpom[j];
      
      emu[i]=1./emu[i];
    }
  }
  //Retardation spectrum
  if (Mm->ip[ipp].tm[im] == creeprs){
    jt = Mm->crrs[i].give_J_E_mu(emu,th_age,tb_age,tb_age_dt,ipp,ido);
  }
  
  //storing retardation coefficients (E_mu of Kelvin chains)into integration point
  switch (Mm->ip[ipp].tm[im]){
  case creepb3:{
    i=Mm->ip[ipp].idm[im];
    Mm->crb3[i].store_emu_eqother(emu,n_ret_times,ipp,ido);
    break;
  }
  case creeprs:{
    i=Mm->ip[ipp].idm[im];
    Mm->crrs[i].store_emu_eqother(emu,n_ret_times,ipp,ido);
    break;
  }
  case creepdpl:{
    i=Mm->ip[ipp].idm[im];
    Mm->crdpl[i].store_emu_eqother(emu,n_ret_times,ipp,ido);
    break;
  }
  default:{
    print_err("\n Unknown material type required in ", __FILE__, __LINE__, __func__);
  }
  }

  //updating of strain increments
  switch (Mm->ip[ipp].tm[im]){
  case creepb3:{
    i=Mm->ip[ipp].idm[im];
    //hidden strains from other
    Mm->crb3[i].give_hidden_strains_eqother(screep,ipp,ido);
    //restoring of previous total overall stresses from eqother
    Mm->crb3[i].give_stresses_eqother(sigmao,ipp,ido);
    //increment of overall stresses
    for (j=0;j<nc;j++){
      sigma[j] = Mm->ip[ipp].stress[j];
      dsigma[j] = sigma[j] - sigmao[j];
    }
    Mm->crb3[i].store_stresses_eqother(sigma,ipp,ido);
    //ageing function 1/v(t)
    inv_v = 1.0;
    q=2.0/3.0;
    //flow creep is already in comliance function
    q4 = 0.0;
    nonlin_func = 1.0;
    break;
  }
   case creeprs:{
    i=Mm->ip[ipp].idm[im];
    //hidden strains from other
    Mm->crrs[i].give_hidden_strains_eqother(screep,ipp,ido);
    //restoring of previous total overall stresses from eqother
    Mm->crrs[i].give_stresses_eqother(sigmao,ipp,ido);
    //increment of overall stresses
    for (j=0;j<nc;j++){
      sigma[j] = Mm->ip[ipp].stress[j];
      dsigma[j] = sigma[j] - sigmao[j];
    }
    Mm->crrs[i].store_stresses_eqother(sigma,ipp,ido);
    //ageing function 1/v(t)
    inv_v = Mm->crrs[i].give_inv_v(tl_age);//aritmetical mid_interval value
    //printf("inv_v = %lf\n",inv_v);
    q=2.0/3.0;
    //flow creep
    q4 = Mm->crrs[i].give_q4();
    nonlin_func = Mm->crrs[i].give_nonlin_func();
    break;
  }
  case creepdpl:{
    i=Mm->ip[ipp].idm[im];
    //hidden strains from other
    Mm->crdpl[i].give_hidden_strains_eqother(screep,ipp,ido);
    //restoring of previous total overall stresses from eqother
    Mm->crdpl[i].give_stresses_eqother(sigmao,ipp,ido);
    //increment of overall stresses
    for (j=0;j<nc;j++){
      sigma[j] = Mm->ip[ipp].stress[j];
      dsigma[j] = sigma[j] - sigmao[j];
    }
    Mm->crdpl[i].store_stresses_eqother(sigma,ipp,ido);
    //ageing function 1/v(t)
    inv_v = 1.0;
    q=2.0/3.0;
    //flow creep is already in comliance function
    q4 = 0.0;
    nonlin_func = 1.0;
    break;
  }
  default:{
    print_err("\n Unknown material type required in ", __FILE__, __LINE__, __func__);
  }
  }
  
  //computes new hidden strains
  creep_hidden_strains (screep,dsigma,emu,mu,n_ret_times,ret_times,tb_age,dt,ss);

  //computes increments of irreversible strains (creep) from previous time step
  for (i=0;i<n_ret_times;i++){
    dj=pow((tb_age + dt)/ret_times[i],q)-pow(tb_age/ret_times[i],q);
    for (j=0; j<nc; j++){
      //deps_cr[j]=deps_cr[j]+(1.-exp(-dj))*screep[j][i];//old
      //new: added effect of non-linear creep and aging function for continuous retardation spectrum
      deps_cr[j] = deps_cr[j] + nonlin_func*((1.-exp(-dj))*(screep[j][i]*inv_v));
    }
  }
  
  //added flow term from continuous retardation spectrum
  if (Mm->ip[ipp].tm[im] == creeprs){
    allocv(nc,deps_help);
    fillv(0.0,deps_help);
    allocm(nc,nc,c);
    fillm(0.0,c);
    unit_compl_matrix(c,mu,ss);
    mxv (c,sigma,deps_help);
    if (Mm->ip[ipp].ssst == planestress)
      {
	deps_help[3] = -mu / (1.0 - mu) * (deps_help[0]+deps_help[1]);
      }	
    
    for (j=0; j<nc; j++){
      deps_cr[j] = deps_cr[j] + nonlin_func*q4*dt/tl_age*deps_help[j];
    }
    destrv(deps_help);
    destrm(c);
  }
  //computes increments of irreversible strains, free shrinkage and stress-induced shrinkage, in actual time step
  switch (Mm->ip[ipp].tm[im]){
  case creepb3:{
    i=Mm->ip[ipp].idm[im];
    Mm->crb3[i].give_deps_free (deps_sh,th_age,tb_age_dt,tb_age,dt,ipp,im,ido);
    Mm->crb3[i].give_deps_stressinduced (deps_ss,th_age,tb_age_dt,tb_age,sigma,ipp,im,ido);

    break;
  }
  case creeprs:{
    i=Mm->ip[ipp].idm[im];
    Mm->crrs[i].give_deps_free (deps_sh,th_age,tb_age_dt,tb_age,dt,ipp,im,ido);
    Mm->crrs[i].give_deps_stressinduced (deps_ss,th_age,tb_age_dt,tb_age,sigma,ipp,im,ido);
    
    break;
  }
  case creepdpl:{
    break;
  }
  default:{
    print_err("\n Unknown material type required in ", __FILE__, __LINE__, __func__);
  }
  }

  //stores new strains into eqother array
  switch (Mm->ip[ipp].tm[im]){
  case creepb3:{
    i=Mm->ip[ipp].idm[im];
    //stores increments of creep-irreversible strains to other
    Mm->crb3[i].store_creepdstrains_eqother(deps_cr,ipp,ido);
    Mm->crb3[i].addirrstrains_eqother (deps_cr,ipp,ido);
    //stores increments of shrinkage-irreversible strains to other
    Mm->crb3[i].store_irrdstrains_eqother(deps_sh,ipp,ido);
    Mm->crb3[i].addirrstrains_eqother (deps_sh,ipp,ido);
    //stores increments of stress-induced shrinkage-irreversible strains to other
    Mm->crb3[i].store_stressirrdstrains_eqother(deps_ss,ipp,ido);
    Mm->crb3[i].addirrstrains_eqother (deps_ss,ipp,ido);
    //stores hidden strains to other
    Mm->crb3[i].store_hidden_strains_eqother(screep,ipp,ido);
    
    // stores humidity
    Mm->crb3[i].store_hum_eqother(ipp,ido);
    // stores temperature
    Mm->crb3[i].store_temp_eqother(ipp,ido);
    break;
  }
  case creeprs:{
    i=Mm->ip[ipp].idm[im];
    //stores increments of creep-irreversible strains to other
    Mm->crrs[i].store_creepdstrains_eqother(deps_cr,ipp,ido);
    Mm->crrs[i].addirrstrains_eqother (deps_cr,ipp,ido);
    //stores increments of shrinkage-irreversible strains to other
    Mm->crrs[i].store_irrdstrains_eqother(deps_sh,ipp,ido);
    Mm->crrs[i].addirrstrains_eqother (deps_sh,ipp,ido);
    //stores increments of stress-induced shrinkage-irreversible strains to other
    Mm->crrs[i].store_stressirrdstrains_eqother(deps_ss,ipp,ido);
    Mm->crrs[i].addirrstrains_eqother (deps_ss,ipp,ido);
    //stores hidden strains to other
    Mm->crrs[i].store_hidden_strains_eqother(screep,ipp,ido);

    // stores humidity
    Mm->crrs[i].store_hum_eqother(ipp,ido);
    // stores temperature
    Mm->crrs[i].store_temp_eqother(ipp,ido);
    break;
  }
  case creepdpl:{
    i=Mm->ip[ipp].idm[im];
    //stores increments of creep-irreversible strains to other
    Mm->crdpl[i].store_creepdstrains_eqother(deps_cr,ipp,ido);
    Mm->crdpl[i].addirrstrains_eqother (deps_cr,ipp,ido);
    //stores hidden strains to other
    Mm->crdpl[i].store_hidden_strains_eqother(screep,ipp,ido);
    break;
  }
  default:{
    print_err("\n Unknown material type required in ", __FILE__, __LINE__, __func__);
  }
  }

  destrv(deps_cr);
  destrv(deps_sh);
  destrv(deps_ss);
  destrv(sigma);
  destrv(sigmao);
  destrv(dsigma);

  destrm(apom);
  destrm(cpom);
  destrv(bpom);
  destrv(ret_times);
  destrv(emu);
}


/**
   function computes new increments of irreversible strains and stresses

   @param ipp - index of integration point
   @param im  - index of
   @param ido - index in array other  

   TKr 07/08/2008
*/
void creep_nlstressesincr (long ipp,long im,long ido)
{
  long i,nc;
  vector deps_cr,deps_sh,deps_ss,dsigma;

  //  number of stres components on element
  nc=Mm->ip[ipp].ncompstr;
  matrix d(nc,nc);

  allocv(nc,deps_cr);
  fillv(0.0,deps_cr);
  allocv(nc,deps_sh);
  fillv(0.0,deps_sh);
  allocv(nc,deps_ss);
  fillv(0.0,deps_ss);
  allocv(nc,dsigma);
  fillv(0.0,dsigma);
  
  //new quantities from eqother array
  switch (Mm->ip[ipp].tm[im]){
  case creepb3:{
    i=Mm->ip[ipp].idm[im];
    //new ages
    Mm->crb3[i].compute_ages (ipp,ido);
    //increments of creep-irreversible strains to other
    Mm->crb3[i].give_creepdstrains_eqother(deps_cr,ipp,ido);
    //increments of shrinkage-irreversible strains to other
    Mm->crb3[i].give_irrdstrains_eqother(deps_sh,ipp,ido);
    //increments of stress-induced shrinkage-irreversible strains to other
    Mm->crb3[i].give_stressirrdstrains_eqother(deps_ss,ipp,ido);
    break;
  }
  case creeprs:{
    i=Mm->ip[ipp].idm[im];
    //new ages
    Mm->crrs[i].compute_ages (ipp,ido);
    //increments of creep-irreversible strains to other
    Mm->crrs[i].give_creepdstrains_eqother(deps_cr,ipp,ido);
    //increments of shrinkage-irreversible strains to other
    Mm->crrs[i].give_irrdstrains_eqother(deps_sh,ipp,ido);
    //increments of stress-induced shrinkage-irreversible strains to other
    Mm->crrs[i].give_stressirrdstrains_eqother(deps_ss,ipp,ido);
    break;
  }
  case creepdpl:{
    i=Mm->ip[ipp].idm[im];
    //new ages
    Mm->crdpl[i].compute_ages (ipp,ido);
    //increments of creep-irreversible strains to other
    Mm->crdpl[i].give_creepdstrains_eqother(deps_cr,ipp,ido);
    break;
  }
  default:{
    print_err("\n Unknown material type required in ", __FILE__, __LINE__, __func__);
  }
  }
  
  //computes increments of creep strains plus shrinkage and stress-induced shrinkage  
  for(i=0;i<nc;i++){
    deps_cr[i]=deps_cr[i]+deps_sh[i]+deps_ss[i];
  }
  
  //computes incremets of stresses from irreversible strain increments
  creep_matstiff (d, ipp, im, ido);
  mxv (d,deps_cr,dsigma);
  
  //storing of actual increments of overall stresses into other
  switch (Mm->ip[ipp].tm[im]){
  case creepb3:{
    i=Mm->ip[ipp].idm[im];
    Mm->crb3[i].store_dstresses_eqother(dsigma,ipp,ido);
    break;
  }
  case creeprs:{
    i=Mm->ip[ipp].idm[im];
    Mm->crrs[i].store_dstresses_eqother(dsigma,ipp,ido);
    break;
  }
  case creepdpl:{
    i=Mm->ip[ipp].idm[im];
    Mm->crdpl[i].store_dstresses_eqother(dsigma,ipp,ido);
    break;
  }
  default:{
    print_err("\n Unknown material type required in ", __FILE__, __LINE__, __func__);
  }
  }
  
  destrv(deps_cr);
  destrv(deps_sh);
  destrv(deps_ss);
  destrv(dsigma);
}


/**
   function returns new increments of irreversible strains and stresses

   @param ipp   - integration point
   @param im    - index of material type for given ip
   @param ido   - index of internal variables for given material in the ipp other array
   @param sigma - %vector of new increments of stresses from irreversible strain increments

   TKr, 07/08/2008 - revised 
*/
void creep_givestressincr (long ipp,long im,long ido,vector &sig)
{
  long i,nc;
  vector dsigma;
  
  //  number of stres components on element
  nc=Mm->ip[ipp].ncompstr;

  allocv(nc,dsigma);
  fillv(0.0,dsigma);
 
  //actual increments of overall stresses from eqother
  switch (Mm->ip[ipp].tm[im]){
  case creepb3:{
    i=Mm->ip[ipp].idm[im];
    Mm->crb3[i].give_dstresses_eqother(dsigma,ipp,ido);
    break;
  }
  case creeprs:{
    i=Mm->ip[ipp].idm[im];
    Mm->crrs[i].give_dstresses_eqother(dsigma,ipp,ido);
    break;
  }
  case creepdpl:{
    i=Mm->ip[ipp].idm[im];
    Mm->crdpl[i].give_dstresses_eqother(dsigma,ipp,ido);
    break;
  }
  default:{
    print_err("\n Unknown material type required in ", __FILE__, __LINE__, __func__);
  }
  }

  for (i=0;i<sig.n;i++){
    sig[i]=dsigma[i];
  }
}
     

/**
   function computes new increments of irreversible strains and stresses

   @param ipp    - integration point
   @param im     - index of material type for given ip
   @param ido    - index of internal variables for given material in the ipp other array
   @param dsigma - %vector of new increments of stresses from irreversible strain increments

   TKr, 07/08/2008 - revised 
*/
void creep_incrtotstresses (long ipp,long im,long ido,vector &dsigma)
{
  long i,j,nc;
  double nu;
  vector eps_old,deps,depstot,deps_cr,deps_sh,deps_ss;
  matrix d;
  strastrestate ss;

  //  number of stres components on element
  nc=Mm->ip[ipp].ncompstr;

  //  stress/strain state
  //planestress=10,planestrain=11,plate=15,axisymm=20,shell=25,spacestress=30
  ss=Mm->ip[ipp].ssst;  

  allocv(nc,eps_old);
  fillv(0.0,eps_old);
  allocv(nc,deps);
  fillv(0.0,deps);
  allocv(nc,depstot);
  fillv(0.0,depstot);
  allocv(nc,deps_cr);
  fillv(0.0,deps_cr);
  allocv(nc,deps_sh);
  fillv(0.0,deps_sh);
  allocv(nc,deps_ss);
  fillv(0.0,deps_ss);

  allocm(nc,nc,d);
  fillm(0.0,d);
  
  //quantities from eqother array
  switch (Mm->ip[ipp].tm[im]){
  case creepb3:{
    i=Mm->ip[ipp].idm[im];    
    //total strains from - previous time step
    Mm->crb3[i].give_strains_eqother(eps_old,ipp,ido);
    //increments of creep-irreversible strains from other
    Mm->crb3[i].give_creepdstrains_eqother(deps_cr,ipp,ido);
    //increments of shrinkage-irreversible strains from other
    Mm->crb3[i].give_irrdstrains_eqother(deps_sh,ipp,ido);
    //increments of stress-induced shrinkage-irreversible strains from other
    Mm->crb3[i].give_stressirrdstrains_eqother(deps_ss,ipp,ido);

    break;
  }
  case creeprs:{
    i=Mm->ip[ipp].idm[im];    
    //total strains from - previous time step
    Mm->crrs[i].give_strains_eqother(eps_old,ipp,ido);
    //increments of creep-irreversible strains from other
    Mm->crrs[i].give_creepdstrains_eqother(deps_cr,ipp,ido);
    //increments of shrinkage-irreversible strains from other
    Mm->crrs[i].give_irrdstrains_eqother(deps_sh,ipp,ido);
    //increments of stress-induced shrinkage-irreversible strains from other
    Mm->crrs[i].give_stressirrdstrains_eqother(deps_ss,ipp,ido);

    break;
  }
  case creepdpl:{
    i=Mm->ip[ipp].idm[im];    
    //total strains from - previous time step
    Mm->crdpl[i].give_strains_eqother(eps_old,ipp,ido);
    //increments of creep-irreversible strains from other
    Mm->crdpl[i].give_creepdstrains_eqother(deps_cr,ipp,ido);
    break;
  }
  default:{
    print_err("\n Unknown material type required in ", __FILE__, __LINE__, __func__);
  }
  }

  //computes increments of total reversible strains from previous time step
  nu = Mm->give_actual_nu(ipp);
  if (Mm->ip[ipp].ssst == planestress)
    {
      Mm->ip[ipp].strain[3] = -nu / (1.0 - nu) * (Mm->ip[ipp].strain[0]+Mm->ip[ipp].strain[1]);
    }
  for (j=0; j<nc; j++){
    depstot[j] = Mm->ip[ipp].strain[j] - eps_old[j];
    deps[j] = depstot[j] - deps_cr[j] - deps_sh[j] - deps_ss[j];
  }

  //  stiffness matrix of material
  creep_matstiff (d, ipp, im, ido);
  //computes new increments of stresses
  mxv (d,deps,dsigma);

  destrv(eps_old);
  destrv(deps);
  destrv(depstot);
  destrv(deps_cr);
  destrv(deps_sh);
  destrv(deps_ss);
}


/**
   function computes new total stresses

   @param ipp - index of integration point
   @param im  - index of
   @param ido - index in array other  

   TKr 07/08/2008 - revised
*/
void creep_nlstresses (long ipp,long im,long ido)
{
  long i,j,nc;
  vector sigma,sigmao,dsigma;

  //  number of stres components on element
  nc=Mm->ip[ipp].ncompstr;

  allocv(nc,sigma);
  fillv(0.0,sigma);
  allocv(nc,sigmao);
  fillv(0.0,sigmao);
  allocv(nc,dsigma);
  fillv(0.0,dsigma);

  creep_incrtotstresses(ipp, im, ido, dsigma);

  //stores new quantities to eqother array
  switch (Mm->ip[ipp].tm[im]){
  case creepb3:{
    i=Mm->ip[ipp].idm[im];
    //old stresses from eqother
    Mm->crb3[i].give_stresses_eqother(sigmao,ipp,ido);
    //stores new total stresses into ip
    for (j=0; j<nc; j++)
      sigma[j] = sigmao[j] + dsigma[j];
    //stores stresses into other
    //Mm->crrs[i].store_stresses_other(sigma,ipp,ido);
    //store stresses into ip
    for (j=0;j<nc;j++){
      Mm->ip[ipp].stress[j] = sigma[j];
    }
    break;
  }
  case creeprs:{
    i=Mm->ip[ipp].idm[im];
    //old stresses from eqother
    Mm->crrs[i].give_stresses_eqother(sigmao,ipp,ido);
    //stores new total stresses into ip
    for (j=0; j<nc; j++)
      sigma[j] = sigmao[j] + dsigma[j];
    //stores stresses into other
    //Mm->crrs[i].store_stresses_other(sigma,ipp,ido);
    //store stresses into ip
    for (j=0;j<nc;j++){
      Mm->ip[ipp].stress[j] = sigma[j];
    }
    break;
  }
  case creepdpl:{
    i=Mm->ip[ipp].idm[im];
    //old stresses from eqother
    Mm->crdpl[i].give_stresses_eqother(sigmao,ipp,ido);
    //stores new total stresses into ip
    for (j=0; j<nc; j++)
      sigma[j] = sigmao[j] + dsigma[j];
    //stores stresses into other
    //Mm->crrs[i].store_stresses_other(sigma,ipp,ido);
    //store stresses into ip
    for (j=0;j<nc;j++){
      Mm->ip[ipp].stress[j] = sigma[j];
    }
    break;
  }
  default:{
    print_err("\n Unknown material type required in ", __FILE__, __LINE__, __func__);
  }
  }

  destrv(dsigma);
  destrv(sigma);
  destrv(sigmao);
}


/**
   function computes hidden strains
   
   @param screep - %vector of hidden strains
   @param sig    - %vector of stress increments
   @param ipp    - index of integration point
   @param im     - index of
   @param ido    - index in array other  

   TKr 07/08/2008 - revised
*/
void creep_hidden_strains (matrix &screep,vector sig,vector emu, double mi, long n_ret_times, vector ret_times, double actualtime, double dt,strastrestate ss)
{
  int i;
  double q,dj,pomt,pp;
  
  q=2./3.;
  if((actualtime+dt) == dt) {
  }
  else{
    pomt=actualtime+dt;
    
    if(ss==bar){
      for (i=0;i<n_ret_times;i++){
	dj=pow(pomt/ret_times[i],q)-pow(actualtime/ret_times[i],q);
	screep[0][i]=exp(-dj)*screep[0][i]+sig[0]*(1.-exp(-dj))/dj/emu[i];
      }
    }
    else if(ss==planestress){
      for (i=0;i<n_ret_times;i++){
	dj=pow(pomt/ret_times[i],q)-pow(actualtime/ret_times[i],q);
	screep[0][i]=exp(-dj)*screep[0][i]+(sig[0]-mi*sig[1])*(1.-exp(-dj))/dj/emu[i];
	screep[1][i]=exp(-dj)*screep[1][i]+(sig[1]-mi*sig[0])*(1.-exp(-dj))/dj/emu[i];
	screep[2][i]=exp(-dj)*screep[2][i]+ sig[2]*2.*(1.+mi)*(1.-exp(-dj))/dj/emu[i];
	screep[3][i]=exp(-dj)*screep[3][i]+(-mi*sig[0]-mi*sig[1])*(1.-exp(-dj))/dj/emu[i];
      }
    }
    else if(ss==planestrain){
      pp=1.+mi;
      for (i=0;i<n_ret_times;i++){
	dj=pow(pomt/ret_times[i],q)-pow(actualtime/ret_times[i],q);
	screep[0][i]=exp(-dj)*screep[0][i]+ pp*(sig[0]*(1.-mi)-mi*sig[1])*(1.-exp(-dj))/dj/emu[i];
	screep[1][i]=exp(-dj)*screep[1][i]+ pp*(sig[1]*(1.-mi)-mi*sig[0])*(1.-exp(-dj))/dj/emu[i];
	screep[2][i]=exp(-dj)*screep[2][i]+ pp*sig[2]*2.   *(1.-exp(-dj))/dj/emu[i];
	screep[3][i]=0.0;
      }
    }
    else if(ss==axisymm){
      for (i=0;i<n_ret_times;i++){
	dj=pow(pomt/ret_times[i],q)-pow(actualtime/ret_times[i],q);
	screep[0][i]=exp(-dj)*screep[0][i]+(sig[0]-mi*sig[1]-mi*sig[2])*(1.-exp(-dj))/dj/emu[i];
	screep[1][i]=exp(-dj)*screep[1][i]+(sig[1]-mi*sig[0]-mi*sig[2])*(1.-exp(-dj))/dj/emu[i];
	screep[2][i]=exp(-dj)*screep[2][i]+(sig[2]-mi*sig[0]-mi*sig[1])*(1.-exp(-dj))/dj/emu[i];
	screep[3][i]=exp(-dj)*screep[3][i]+ sig[3]*(1.+mi)   *(1.-exp(-dj))/dj/emu[i];
      }
    }
    else if(ss==spacestress){
      for (i=0;i<n_ret_times;i++){
	dj=pow(pomt/ret_times[i],q)-pow(actualtime/ret_times[i],q);
	screep[0][i]=exp(-dj)*screep[0][i]+(sig[0]-mi*sig[1]-mi*sig[2])*(1.-exp(-dj))/dj/emu[i];
	screep[1][i]=exp(-dj)*screep[1][i]+(sig[1]-mi*sig[0]-mi*sig[2])*(1.-exp(-dj))/dj/emu[i];
	screep[2][i]=exp(-dj)*screep[2][i]+(sig[2]-mi*sig[0]-mi*sig[1])*(1.-exp(-dj))/dj/emu[i];
	screep[3][i]=exp(-dj)*screep[3][i]+ sig[3]*(1.+mi)   *(1.-exp(-dj))/dj/emu[i];
	screep[4][i]=exp(-dj)*screep[4][i]+ sig[4]*(1.+mi)   *(1.-exp(-dj))/dj/emu[i];
	screep[5][i]=exp(-dj)*screep[5][i]+ sig[5]*(1.+mi)   *(1.-exp(-dj))/dj/emu[i];
      }
    }
  }
}


/**
   function returns total irreversible strains

   @param ipp   - integration point
   @param im    - index of material type for given ip
   @param ido   - index of internal variables for given material in the ipp other array
   @param epscr - %vector of irreversible strains

   TKr, 07/08/2008 - revised 

*/
void creep_giveirrstrains (long ipp, long im, long ido, vector &epscr)
{
  long i;
  
  switch (Mm->ip[ipp].tm[im]){
  case creepb3:{
    i=Mm->ip[ipp].idm[im];
    Mm->crb3[i].giveirrstrains_eqother (ipp,ido,epscr);
    break;
  }
  case creeprs:{
    i=Mm->ip[ipp].idm[im];
    Mm->crrs[i].giveirrstrains_eqother (ipp,ido,epscr);
    break;
  }
  case creepdpl:{
    i=Mm->ip[ipp].idm[im];
    Mm->crdpl[i].giveirrstrains_eqother (ipp,ido,epscr);
    break;
  }
  default:{
    print_err("\n Unknown material type required in ", __FILE__, __LINE__, __func__);
  }
  }
}


/**
   function returns number of retardation times

   @param ipp    - integration point
   @param im     - index of material type for given ip

   TKr, 07/08/2008 - revised 
*/
long creep_number_rettimes (long ipp,long im)
{
  long i,n_ret_times;

  //creep function
  switch (Mm->ip[ipp].tm[im]){
  case creepb3:{
    i=Mm->ip[ipp].idm[im];
    
    //number of retardation times
    n_ret_times = Mm->crb3[i].give_nret_time();
    break;
  }
  case creeprs:{
    i=Mm->ip[ipp].idm[im];
    
    //number of retardation times
    n_ret_times = Mm->crrs[i].give_nret_time();
    break;
  }
  case creepdpl:{
    i=Mm->ip[ipp].idm[im];
    
    //number of retardation times
    n_ret_times = Mm->crdpl[i].give_nret_time();
    break;
  }
  default:{
    print_err("\n Unknown material type required in ", __FILE__, __LINE__, __func__);
  }
  }
  
  return(n_ret_times);
}

/**
  function returns number of eqother components

   @param ipp    - integration point
   @param im     - index of material type for given ip

   TKr, 07/08/2008 - revised 
*/
long creep_ncompo (long ipp,long im)
{
  long i,ncompo;

  //creep function
  switch (Mm->ip[ipp].tm[im]){
  case creepb3:{
    i=Mm->ip[ipp].idm[im];
    
    ncompo = Mm->crb3[i].give_nceqother (ipp);
    break;
  }
  case creeprs:{
    i=Mm->ip[ipp].idm[im];
    
    ncompo = Mm->crrs[i].give_nceqother (ipp);
    break;
  }
  case creepdpl:{
    i=Mm->ip[ipp].idm[im];
    
    ncompo = Mm->crdpl[i].give_nceqother (ipp);
    break;
  }
  default:{
    print_err("\n Unknown material type required in ", __FILE__, __LINE__, __func__);
  }
  }
  
  return(ncompo);
}



/**
   function assembles stiffness %matrix of material
   
   @param d - stiffness %matrix of material
   @param ssst - strain/stress state
*/
void unit_compl_matrix (matrix &c,double nu,strastrestate ssst)
{
  fillm(0.0,c);
  
  switch (ssst){
  case bar:{
    c[0][0] = 1.0;
    break;
  }
  case plbeam:{
    c[0][0] = 1.0;
    c[1][1] = 1/(1/2.0/(1.0+nu));
    c[2][2] = 1.0;
    break;
  }
  case spacebeam:{
    c[0][0] = 1;
    c[1][1] = 1.0/(1/2.0/(1.0+nu));
    c[2][2] = 1.0/(1/2.0/(1.0+nu));
    c[3][3] = 1.0/(1/2.0/(1.0+nu));
    c[4][4] = 1;
    c[5][5] = 1;
    break;
  }
  case planestress:{
    double g;
    
    g = 1.0/(2*(1.0+nu))*2.0*(1+nu);
    
    c[0][0] = g;     c[0][1] = -nu*g; c[0][2] = 0.0;
    c[1][0] = -nu*g; c[1][1] = g;     c[1][2] = 0.0;
    c[2][0] = 0.0;   c[2][1] = 0.0;   c[2][2] = 1;
    
    //tady doplnit??!!

    break;
  }
  case planestrain:{
    double g;
    
    g = (1.0 - nu)/2.0*2.0*(1+nu);
    
    c[0][0] = g;       c[0][1] = -nu/2.0; c[0][2] = 0.0;
    c[1][0] = -nu/2.0; c[1][1] = g;       c[1][2] = 0.0;
    c[2][0] = 0.0;     c[2][1] = 0.0;     c[2][2] = 1;

    break;
  }
  case spacestress:{
    c[0][0]=1;         c[0][1]=-nu;      c[0][2]=-nu;
    c[1][0]=c[0][1];   c[1][1]=c[0][0];  c[1][2]=c[0][1];
    c[2][0]=c[0][1];   c[2][1]=c[0][1];  c[2][2]=c[0][0];
    c[3][3]=2.0*(1.0+nu);
    c[4][4]=2.0*(1.0+nu);
    c[5][5]=2.0*(1.0+nu);
    break;
  }
  default:{
    print_err("unknown number of components of stress tensor is required", __FILE__, __LINE__, __func__);
  }
  }
}
