#include "creepdam.h"
#include "creep.h"
#include "creep_b3.h"
#include "creep_rspec.h"
#include "creepb.h"
#include "creep_dpl.h"
#include "matrix.h"
#include "vector.h"
#include "elastisomat.h"
#include "global.h"
#include "intpoints.h"
#include "vecttens.h"
#include <math.h>

/**
  This constructor inializes attributes to zero values.
*/
creepdam::creepdam (void)
{

}

/**
  This destructor is only for the formal purposes.
*/
creepdam::~creepdam (void)
{

}

/**
  This function computes material stiffnes %matrix.

  @param d - allocated matrix structure for material stiffness %matrix
  @param ipp - integration point number
  @param im - material index
  @param ido - index of internal variables for given material in the ipp eqother array


*/
void creepdam::matstiff (matrix &d,long ipp,long im, long ido)
{
  long ncompo = Mm->givencompeqother(ipp, im+1); 
  Mm->matstiff(d, ipp, im+2, ido+1+ncompo);
}


/**
  This function computes correct stresses in the integration point and stores
  them into ip stress array. It was used for Hinkley computations and supposes creep 
  strains to be read from backup file.
  
  @param ipp - integration point pointer
  7.10.2001

void creepdam::nlstresses (long ipp)
{
  long i;
  long ncomp = Mm->ip[ipp].ncompstr;
  vector epsback(ncomp);


  //  backup of the total strains
  for (i=0;i<ncomp;i++){
    epsback[i] = Mm->ip[ipp].strain[i];
  }

  // compute creep - actually it is supposed the creep strains are read from backup file
  //                 to the eqother array.
  //  initial values of elastic strain for damage = total strain - creep strain
  for (i=0;i<ncomp;i++){
    Mm->ip[ipp].strain[i] -= Mm->ip[ipp].eqother[i];
  }
  
  // compute damage, it has index 1, eqother values for damage starts at ncomp index
  Mm->computenlstresses(ipp, 1, 2*ncomp);
  
  //  recovery of the total strains
  for (i=0;i<ncomp;i++){
    Mm->ip[ipp].strain[i] = epsback[i];
  }

}
*/




/**
  This function computes stresses increments fro creep model
  in the integration point and stores them into ip stress array.
  
  @param ipp - integration point pointer
  @param im  - index of the material in the ipp tm and idm arrays. The standard value is zero.
  @param ido - index of internal variables for given material in the ipp other array

  Created by Tomas Koudelka  7.2008
*/
void creepdam::nlstressesincr(long ipp, long im, long ido)
{
  long i;
  long ncomp = Mm->ip[ipp].ncompstr;
  double nu,omegao;

  if (Mm->ip[ipp].ssst == planestress)
  {
    nu = Mm->give_actual_nu(ipp);
    Mm->ip[ipp].strain[3] = -nu / (1.0 - nu) * (Mm->ip[ipp].strain[0]+Mm->ip[ipp].strain[1]);
  }

  // compute creep , it has index1, creep eqother values starts from 0.
  Mm->computenlstressesincr(ipp, im+1, ido+1);
  // damage from the previous equlibriated time step
  omegao = Mm->ip[ipp].eqother[ido];
  for (i=0;i<ncomp;i++)
    Mm->ip[ipp].stress[i] *= (1.0-omegao);
  
  return;
}



/**
  This function computes correct stresses in the integration point and stores
  them into ip stress array.
  
  @param ipp - integration point pointer
  @param im  - index of the material in the ipp tm and idm arrays. The standard value is zero.
  @param ido - index of internal variables for given material in the ipp other array

  7.10.2001
*/
void creepdam::nlstresses (long ipp, long im, long ido)
{
  long i, idem;
  long ncomp = Mm->ip[ipp].ncompstr;
  long ncompo;
  double nu,omega=0.0;
  double omegao;
  vector epsback(ncomp);
  vector sigmao(ncomp);
  vector dsigma(ncomp);
  vector epscr(ncomp);


  if (Mm->ip[ipp].ssst == planestress)
  {
    idem = Mm->ip[ipp].gemid();
    nu = Mm->eliso[Mm->ip[ipp].idm[idem]].nu;
    Mm->ip[ipp].strain[3] = -nu / (1.0 - nu) * (Mm->ip[ipp].strain[0]+Mm->ip[ipp].strain[1]);
  }
  // damage from the previous equlibriated time step
  omegao = Mm->ip[ipp].eqother[ido];

  //  backup of the total strains
  for (i=0;i<ncomp;i++)
    epsback[i] = Mm->ip[ipp].strain[i];
  //  initial values of elastic strain for damage = total strain - creep strain
  Mm->giveirrstrains(ipp, im+1, ido+1, epscr);
  for (i=0;i<ncomp;i++)
    Mm->ip[ipp].strain[i] -= epscr[i];

  // number of components in eqother array for creep 
  ncompo = Mm->givencompeqother(ipp, im+1); 
  // compute damage, it has index 2, eqother values for damage starts at ncompo index
  if ((Mm->ip[ipp].hmt & 2) && (Mp->nonlocphase == 2))// nonlocal damage
    Mm->compnonloc_nlstresses(ipp, im+2, ido+1+ncompo);
  else
    Mm->computenlstresses(ipp, im+2, ido+1+ncompo);
  omega = Mm->givedamage(ipp,im+2,ido+1+ncompo);
  //  recovery of the total strains and total stress increment
  for (i=0;i<ncomp;i++)
    Mm->ip[ipp].strain[i] = epsback[i];

  creep_incrtotstresses (ipp, im+1, ido+1, dsigma);
  switch (Mm->ip[ipp].tm[im+1])
  {
    case creepb3:
      //Mm->crb3[i].give_stresses_eqother(sigmao,ipp,ido+1);
      break;
    case creeprs:
      Mm->crrs[i].give_stresses_eqother(sigmao,ipp,ido+1);
      break;
    case creepdpl:
      //Mm->crdpl[i].give_stresses_eqother(sigmao,ipp,ido+1);
      break;
    default:
      print_err("unknown material type is required",__FILE__,__LINE__,__func__);
  }
  for (i=0;i<ncomp;i++)
  {
    if (1.0-omegao > 1.0e-6)
      Mm->ip[ipp].stress[i] = (1.0-omega)*(sigmao[i]/(1.0-omegao) + dsigma[i]);
    else
      // full damage was reached => difference between omega and omegao are negligable
      Mm->ip[ipp].stress[i] = (1.0-omega)*dsigma[i] + sigmao[i];
  }
    //    Mm->ip[ipp].stress[i] = sigmao[i]+(1.0-omega)*dsigma[i];
    //Mm->ip[ipp].stress[i] = sigmao[i]+sigmao[i]*(omegao-omega)+(1.0-omega)*dsigma[i];
  
  if (ipp == 0)
  {
//    fprintf(Out, "\nCas %e: sigmao=%e, sigmao*(omegao-omega)=%e, (1-omega)*dsigma=%e, omega=%e, omegao=%e",
    fprintf(Out, "\n%e %e %e %e %e %e %e",
	    Mp->time, Mm->give_actual_ym(ipp), sigmao[0], sigmao[0]*(omegao-omega), (1.0-omega)*dsigma[0], omega, omegao);
  }
  Mm->ip[ipp].other[ido] = omega;
}


/**
  This function updates values in the other array reached in the previous equlibrium state to
  values reached in the new actual equilibrium state.

  @param ipp - integration point number in the mechmat ip array.
  @param im - material index
  @param ido - index of internal variables for given material in the ipp other array

*/
void creepdam::updateval (long ipp, long im, long ido)
{
  //  update values of plasticity model
  //  long ncompo = 2*Mm->ip[ipp].ncompstr; // for Hinkley
  //  Mm->updateipvalmat (ipp,1,ncompo); // for Hinkley
  Mm->ip[ipp].eqother[ido] = Mm->ip[ipp].other[ido];
  long ncompo = Mm->givencompeqother(ipp, im+1); 
  Mm->updateipvalmat (ipp,im+1,ido+1); 
  Mm->updateipvalmat (ipp,im+2,ido+1+ncompo);  
}



/**
   Function initializes eqother array with initial initial values.
   Actual value of array Mm->tempr[ipp] is taken as initial temperature.
   
   @param ipp - integration point pointer
   @param im - material index
   @param ido - index of internal variables for given material in the ipp other array

   7.6.2005, TKo
*/
void creepdam::initvalues (long ipp, long im, long ido)
{
  long ncompo = Mm->givencompeqother(ipp, im+1); 
  Mm->initvalues(ipp,im+1,ido+1); 
  Mm->initvalues(ipp,im+2,ido+1+ncompo); 
}



/**
  This function returns the value of tensile strength

  @param ipp - integration point number in the mechmat ip array.
  @param im - material index
  @param ido - index of internal variables for given material in the ipp other array
  
  TKo
*/
double creepdam::give_actual_ft (long ipp, long im, long ido)
{
  double ft;
  ft = Mm->give_actual_ft(ipp, im+1, ido+1);
  return ft;
}



/**
  This function returns the value of compressive strength

  @param ipp - integration point number in the mechmat ip array.
  @param im - material index
  @param ido - index of internal variables for given material in the ipp other array
  
  10.2008 TKo
*/
double creepdam::give_actual_fc (long ipp, long im, long ido)
{
  double fc;  
  long ncompo_cr = Mm->givencompeqother(ipp, im+1);
  // strength is obtained from damage material;
  fc = Mm->give_actual_fc(ipp, im+2, ido+1+ncompo_cr);
  return fc;
}
