#ifndef FLSUBDOM_H
#define FLSUBDOM_H

#include <stdio.h>
#include "alias.h"


/**
   class flsubdom serves for problems with floating subdomains
   solved by the FETI method
   
   JK, 20.6.2006
*/

class flsubdom
{
 public:
  flsubdom ();
  ~flsubdom ();
  
  void initialization (long ndofs,long esnrbm,double *rhs);
  void evector (double *f);
  void g_matrix (void);
  void feti_projection (double *v);
  void pmcg (double *rhs,FILE *out);
  void lagrmultdispl (double *d,double *f);
  void nonlinlagrmultdispl (double *d,double *f);
  void solve_lin_alg_system (double *lhs,double *rhs);
  
  void add_mult (double dlambda);
  void mult_correction ();
  void factorize ();

  
  ///  estimated number of rigid body motions
  long enrbm;
  ///  number of rigid body motions
  long nrbm;
  ///  number of DOFs
  long ndof;
  ///  number of Lagrange multipliers
  long nlm;
  ///  array for list of dependent rows and columns
  long *de;

  ///  array for rigid body motions
  double *rbm;
  ///  %matrix G
  double *gg;
  ///  %matrix (G^T G)^{-1}
  double *igg;
  ///  %vector e
  double *e;
  ///  %vector of Lagrange multipliers
  ///  it contains increments of multipliers in nonlinear problems
  double *w;
  ///  %vector of total Lagrange multipliers
  ///  it is used only in nonlinear problems
  double *tw;
  
  
  ///  number of changes in %vector of Lagrange multipliers
  long nch;
  ///  compliances
  double *compli;
  
  
  double *mu,*muo,*dmu,*ddmu;
};

#endif
