#ifndef ORTODAM_H
#define ORTODAM_H

#include "iotools.h"
#include "alias.h"
#include "strretalg.h"
struct matrix;
struct vector;
struct atsel;

/**
  This class defines scalar isotropic damage material model.
  The different type of norms for the computing parameters of
  the damage function can be used.
  The order of internal variables is following :
  0 - actual value of param At
  1 - actual value of param Ac
  3..5 - reached values of principal damage parameters for tension
  6..8 - reached values of principal damage parameters for compression
  9..14 - damage tensor for tension
  15..18 - cracks openings computed from tensile damage tensor
*/
class ortodam
{
 public:
  ortodam (void);
  ~ortodam (void);
  void   read (XFILE *in);
  double damdrvforce_vol(double nu, vector &peps);
  void   damdrvforce_dev(double nu, vector &peps, vector &pyt, vector &pyc);
  void   loadfunc(long ipp, double nu, vector &peps, vector &damt, vector &damc, double aat, double aac, vector &lft, vector &lfc);
  double brittle_damage(long ipp, double y, double e, double f, double uf);
  void   princ_dam(long ipp, vector &peps, double aat, double aac, vector &pdamt, vector &pdamc);
  void   give_actual_param_a(long ipp, long ido, double &aat, double &aac);
  double give_actual_ft(long ipp);
  double give_actual_fc(long ipp);
  void   initvalues(long ipp, long ido);
  void   matstiff (matrix &d,long ipp,long ido);
  void   pelmatstiff (long ipp, matrix &d,double e, double nu);
  void   elmatstiff (matrix &d,long ipp);
  void   nlstresses (long ipp, long im, long ido);
  void   updateval (long ipp,long im,long ido);


  /// type of damage evolution function
  dam_evolfunc damevf;
  /// tensile strength
  double ft;
  /// initial crack opening u_ft or eps_ft - controls initial slope of softening branch for tension
  double uft;
  /// compressive strength
  double fc;
  /// initial crack opening u_fc or eps_fc - controls initial slope of softening branch for compression
  double ufc;
  /// parameters for Newton tangent method for damage parameter computation for brittle evolution fucntion and correction of dissipated energy
  strretalg sra;
  /// correction of disipated energy switch
  corr_disip_en cde;
  /// material parameter At for tension
  double at;  
  /// material parameter Bt for tension
  double bt;  
  /// initial treshold for dimensionless damage driving force Y0 for tension
  double y0t;  
  /// material parameter Ac for compression
  double ac;  
  /// material parameter Bc for compression
  double bc;  
  /// initial treshold for dimensionless damage driving force Y0 for compression
  double y0c;  
  /// fracture energy of damage for tension
  double gft;
  /// fracture energy of damage for compression
  double gfc;  
  
};

#endif
