#ifndef SCALDAMTIME_H
#define SCALDAMTIME_H

#include <stdio.h>
#include "alias.h"
#include "strretalg.h"
struct matrix;
struct vector;
struct atsel;

/**
  This class defines scalar isotropic damage material model for time dependent problems.
  The different type of norms for the computing parameters of the damage function can be used.
  Order of internal variables in the other array :
  ------------------------------------------------------------------------------------
  0 .. ncompstr-1          : previous total strains
  ncompstr .. 2*ncompstr-1 : increment of total strains
  2*ncompstr               : previous equivalent strain
  2*ncompstr+1             : previous damage
  2*ncompstr+2             : actual damaga
  ------------------------------------------------------------------------------------
  Total number of components of other array : 2*ncompstr+3

*/
class scaldamtime
{
 public:
  scaldamtime (void);
  ~scaldamtime (void);
  void read (FILE *in);
  void damfuncpar(long ipp, vector &eps, vector &kappa);
  double damfunction(long ipp, vector &kappa);
  void der_damfuncpar(long ipp, vector &eps, vector &kappa, vector &depseq);
  double der_damfunction(long ipp, vector &kappa);
  void matstiff (matrix &d,long ipp,long ido);
  void elmatstiff (matrix &d,long ipp);
  void nlstresses (long ipp, long im, long ido);
  double epsefunction (long ipp);
  void changeparam (atsel &atm,vector &val);
  double givedamage(long ipp, long ido);
  double give_actual_ft (long ipp, long im, long ido);

  /// max effective strain at peak
  double indam;
  /// tensile strength
  double st;
  /// determines the softening -> corresponds to crack opening (not strain) when tension stress vanishes
  double uf;
  /// parameter for energy norm
  double c;
  /// parameter for von Mises norm - ratio of compression and tensile strength
  double kf;
  /// type of function for evaluation damage funtion parameter
  paramf_type ftype;
  /// correction of disipated energy switch
  corr_disip_en cde;
  ///  stress return algorithm
  strretalg sra;
};

#endif
